#!/usr/bin/env python3
#
#  (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from pprint import pprint
from ttproto.utils.pure_pcapy import DLT_IEEE802_15_4
from ttproto.core.lib.all import Ieee802154
from ttproto.utils.pcap_filter import remove_first_bytes, openwsn_profile_filter, finterop_tun_profile_filter, \
    remove_first_frames

if __name__ == '__main__':
    # from ttproto.core.dissector import Dissector
    #
    # #filename = 'tests/test_dumps/6lowpan/echo_req_and_reply_and_other_packets_with_openmote_sniffer.pcap'
    #
    # openwsn_profile_filter(filename)
    #
    # print('starting dissection')
    # dissector = Dissector('tmp/temp.pcap')
    #
    # for s in dissector.summary():
    #     print(s)
    #
    # print("\n Ieee802154 frames: \n")
    #
    # for s in dissector.summary(protocol = Ieee802154):
    #     print(s)

    from ttproto.core.dissector import Capture, Dissector

    # filename = '/Users/fsismondi/TD_COAP_CORE_01.pcap'
    # # dissect with no modifs
    # dissector = Dissector(filename)
    #
    # # dissect with filter
    # finterop_tun_profile_filter(filename)
    # #dissector = Dissector('tmp/temp.pcap')
    # print(dissector.summary())
    # print('#####')
    # print(dissector.dissect())

    filename1 = 'tmp/DLT_RAW_1.pcap'
    # dissect with no modifs
    c1 = Capture(filename1)

    # dissect with filter
    remove_first_frames(
        number_of_frames_to_skip=2,
        pcap_filename=filename1,
        new_pcap_filename='tmp/temp.pcap',
    )
    c2 = Capture('tmp/temp.pcap')

    print('#####')
    pprint(c1.summary())
    print('#####')
    pprint(c2.summary())



#!/usr/bin/env python3

#from ttproto.ts_coap.proto_specific import CoAPTestcase
# from ttproto.ts_coap.proto_templates import *
from ttproto.ts_coap.testcases.td_coap_core_22 import TD_COAP_CORE_22

class TD_COAP_CORE_27 (TD_COAP_CORE_22):
	"""Identifier:
TD_COAP_CORE_27
Objective:
Perform GET transaction with responses containing the ETag option and requests
containing the If-Match option (CON mode)
Configuration:
CoAP_CFG_03
References:
[1] clause 5.8.1, 5.10.7,5.10.9,12.1.12, 8.2.2,8.2.1,10.2.2,11.2

Pre-test
conditions:
•	Proxy is configured as a reverse-proxy for the server
•	Proxy’s cache is cleared
•	Server should offer a /validate resource with resource content is not empty
•	Client & server supports ETag option and If-Match option

Test Sequence:
Step
Type
Description
Preamble: client gets the resource


1
Stimulus
Client is requested to send a confirmable GET request to
proxy

2
Check
Proxy forwards the request to server


3
Check
Forwarded request must contain:
•	Type = 0 (CON)
•	Code = 1 (GET)

4
Check
Server sends response containing:
•	Code = 69 (2.05 content)
•	Option type = ETag
•	Option value = an arbitrary ETag value
•	Not empty payload

5
Check
Proxy forwards the response to client
Part A: single update

6
Stimulus
Client is requested to send a confirmable PUT request to
Proxy

7
Check
Sent request must contain:
•	Type = 0 (CON)
•	Code = 3 (PUT)
•	Option Type=If-Match
•	Option value=ETag value received in step 4
•	An arbitrary payload (which differs from the payload
received in step 3)

8
Verify
Proxy forwards the request to servers resource & server
updates the resource


9
Check
Server sends response containing:
•	Code = 68 (2.04 Changed)
•	Option type = ETag
•	Option value = an arbitrary ETag value which differs from
the ETag received in step 4


10
Check
Proxy forwards the response to client

11
Check
Forwarded response contains:
•	Code = 68 (2.04 Changed)
•	Option type = ETag
•	Option value = same ETag value found in step 8


12
Verify
Client displays the response
Part B: concurrent updates

13
Stimulus
Update the content of the server’s resource from a CoAP
client

14
Stimulus
Client is requested to send s confirmable PUT request to
proxy so as to perform an atomic update

15
Check
Sent request must contain:
•	Type = 0 (CON)
•	Code = 3 (PUT)
•	Option Type=If-Match
•	Option value=ETag value received in step 8
An arbitrary payload (which differs from the previous
payloads)

16
Check
Proxy forwards the request to server’s resource

17
Check
Sent request must contain:
•	Type = 0 (CON)
•	Code = 3 (PUT)
•	Option Type=If-Match
•	Option value=same ETag value found in step 14
An arbitrary payload (which differs from the previous
payloads)

18
Check
Server sends response containing:
•	Code = 140 (4.12 Precondition Failed)


19
Verify
Proxy forwards the response to client

20
Check
Response contains:
Code = 140 (4.12 Precondition Failed)

21
Verify
Client displays the response
	"""
	reverse_proxy = True





#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import re

from ttproto.core.exceptions import Error
from ttproto.core.data import Value, BidictValueType
from ttproto.core.typecheck import *
from ttproto.core.lib.inet.meta import *
from ttproto.core.lib.inet.basics import *
from ttproto.core.lib.ports import socat
from ttproto.core.lib.ethernet import ethernet_type_bidict

__all__ = [
    'LinuxCookedCapture',
    'NullLoopback',
]


class LinuxCookedCapture(
        metaclass=InetPacketClass,
        fields=[
            ("PacketType", "pty", UInt16),
            ("AddressType", "aty", UInt16),
            ("AddressLength", "aln", UInt16),
            ("Address", "adr", Bytes8),
            ("Protocol", "pro", UInt16, InetType(ethernet_type_bidict, "Payload")),
            # FIXME: values below 1537 are treated differently (see wireshark dissector packet-sll.c)
            ("Payload", "pl", Value),
        ],
        descriptions={
            "PacketType": {
                0: "unicast to us",
                1: "broadcast",
                2: "multicast",
                3: "unicast to another host",
                4: "sent by us",
            }
        }):
    def describe(self, desc):
        return self.describe_payload(desc)


# Null Loopback cooked L2 generated in BSD systems

# check packet-null.c in wireshark for more info

# BSD AF_ values (family type values):
# BSD_AF_INET		2
# BSD_AF_ISO		7
# BSD_AF_APPLETALK	16
# BSD_AF_IPX		23
# BSD_AF_INET6_BSD	24	/* NetBSD, OpenBSD, BSD/OS */
# BSD_AF_INET6_FREEBSD	28	/* FreeBSD, DragonFly BSD */
# BSD_AF_INET6_DARWIN	30	/* OS X, iOS, anything else Darwin-based */

encap_type_bidict = BidictValueType(0, UInt32)


class NullLoopback(
        metaclass=InetPacketClass,
        fields=[
            ("AddressFamily", "AF", UInt8, InetType(encap_type_bidict, "Payload")),
            ("ProtocolFamily", "PF", UInt24),
            ("Payload", "pl", Value),
        ],
        descriptions={
            "AddressFamily": {
                2: "BSD INET",
                7: "BSD ISO",
                16: "BSD APPLETALK",
                23: "BSD IPX",
                24: "BSD INET6 BSD",
                28: "BSD INET6 FREEBSD",
                30: "BSD INET6 DARWIN"
            }

        }):
    def describe(self, desc):
        return self.describe_payload(desc)



#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


from collections import OrderedDict
from os import path
import logging
import re

from ttproto.core.exceptions import Error, ReaderError, UnknownField
from ttproto.core.data import Data, Message
from ttproto.core.list import ListValue
from ttproto.core.packet import Value, PacketValue
from ttproto.core.typecheck import typecheck, list_of, optional, anything, either
from ttproto.core.lib.all import *
from ttproto.core.lib.inet.meta import InetPacketValue
from ttproto.core.lib.readers.pcap import PcapReader

log = logging.getLogger('[dissection]')
log.propagate = True  # so AMQP handler (if attached by ancestor) emits logs into the bus
log.setLevel(level=logging.WARNING)

__all__ = [
    'get_dissectable_protocols',
    'is_protocol',
    'is_layer_value',
    'ProtocolNotFound',
    'Frame',
    'Dissector',
    'Capture',
]


def add_subclass(impl_list, new_class):
    subclasses = new_class.__subclasses__()
    # print(new_class.__name__ + ':')
    # print(subclasses)
    impl_list += subclasses
    # print('#####################')
    for subclass in subclasses:
        add_subclass(impl_list, subclass)


@typecheck
def get_dissectable_protocols() -> list_of(type):
    """
    Return list of the implemented protocols (PacketValue classes)
    :return: Implemented protocols
    :rtype: [type]
    """
    # Just directly get the PacketValue and InetPacketValue subclasses
    implemented_protocols = []
    add_subclass(implemented_protocols, PacketValue)
    # Remove the InetPacketValue class
    implemented_protocols.remove(InetPacketValue)
    return implemented_protocols


@typecheck
def is_protocol(arg: anything) -> bool:
    """
    Check if a parameter is a valid protocol.
    This function is used for the typechecker decorator.

    :param arg: The object to check
    :type arg: anything

    :return: True if a valid protocol, False if not
    :rtype: bool
    """
    return all((
        arg is not None,
        type(arg) == type,
        arg in get_dissectable_protocols()
    ))


@typecheck
def is_layer_value(arg: anything) -> bool:
    """
    Check if a parameter is a valid layer value.
    This function is used for the typechecker decorator.

    :param arg: The object to check
    :type arg: anything

    :return: True if a valid layer value, False if not
    :rtype: bool
    """
    return all((
        arg is not None,
        isinstance(arg, Value)
    ))


class ProtocolNotFound(Error):
    """
    Error thrown when a protocol isn't found in a frame
    """
    pass


class Frame:
    """
        Class to represent a frame object
    """

    @typecheck
    def __init__(
            self,
            id: int,
            pcap_frame: (float, Message, optional(Exception))
    ):
        """
        The init function of the Frame object

        :param id: The id of the current frame
        :param pcap_frame: The frame tuple got from reading the PcapReader
        :type id: int
        :type pcap_frame: (float, Message, Exception)
        """

        # Put the different variables of it
        self.__id = id
        log.debug("[Frame Init] New Frame id: %d" % id)

        # Get the 3 values of a frame given by the PcapReader
        # ts: Its timestamp value (from the header)
        # msg: Its message read directly from bytes (can be decoded)
        # exc: Exception if one occured
        self.__timestamp, self.__msg, self.__error = pcap_frame

        # Put its dictionnary representation and its summary as not done yet
        self.__dict = None
        self.__summary = None

    @typecheck
    def __contains__(self, protocol: is_protocol) -> bool:
        """
        Put the 'in' keyword to check if a protocol is contained in frame

        :param protocol:  Protocol to check
        :type protocol: type

        :raises TypeError: If protocol is not a valid protocol class

        :return: True if the protocol is in the protocol stack of the frame
        :rtype: bool
        """

        # Check the protocol is one entered
        if not is_protocol(protocol):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        # Get current value
        value = self.__msg.get_value()

        # Parse the whole protocol stack
        while True:

            # If the protocol is contained into it
            if isinstance(value, protocol):
                return True

            # Go to the next layer
            try:
                value = value['pl']
                continue

            # If none found, leave the loop
            except (KeyError, TypeError, UnknownField):
                pass
            break

        # Protocol not found into it
        return False

    @typecheck
    def __repr__(self) -> str:
        """
        Little function to display a frame object

        :return: A string representing this frame object
        :rtype: str
        """
        return "<Frame %3d: %s>" % (self.__id, self.__msg.summary())

    @property
    def timestamp(self):
        return self.__timestamp

    @property
    def message(self):
        return self.__msg

    @property
    def error(self):
        return self.__error

    # @typecheck
    def __value_to_list(
            self,
            l: list,
            value: Value,
            extra_data: optional(str) = None,
            layer_dict: optional(dict) = None,
            is_option: optional(bool) = False
    ):
        """
        An utility function to parse recursively packets' data

        :param l: The list in which we put the values parsed
        :param value: The value to store
        :param extra_data: The name of the field to save value into dict
        :param layer_dict: The dict in which we will write the value
        :param is_option: To know if the value to write is an option one or not
        :type l: list
        :type value: Value
        :type extra_data: str
        :type layer_dict: dict
        :type is_option: bool
        """
        try:
            # Points to packet
            # print("dissecting value: " + str(value) + " || type : " + str(type(value)))
            if isinstance(value, PacketValue):

                # Prepare the storage dict
                od = OrderedDict()

                # If an option
                if is_option:
                    od['Option'] = value.get_variant().__name__

                # If a protocol value
                else:
                    # log.debug(' Protocol header:  ' + str(value.get_variant().__name__))
                    od['_type'] = 'protocol'
                    od['_protocol'] = value.get_variant().__name__

                l.append(od)

                i = 0
                for f in value.get_variant().fields():
                    self.__value_to_list(l, value[i], f.name, od)
                    i += 1

            # Points to list value
            elif isinstance(value, ListValue):

                prot_options = []
                for i in range(0, len(value)):
                    self.__value_to_list(prot_options, value[i], is_option=True)
                # log.debug(' options:    || value : ' + str(prot_options))
                layer_dict['Options'] = prot_options

            # If it's a single field
            else:
                # log.debug(' field:  ' + str(extra_data) + '|| value : ' + str(value))
                layer_dict[extra_data] = str(value)

        except TypeError as e:
            # log.error(e.__traceback__)
            log.error(e, exc_info=True)
            log.error(
                'extra_data:  ' + str(extra_data) + '|| value : ' + str(value) + '|| layerDict : ' + str(layer_dict))

    @typecheck
    def dict(self) -> OrderedDict:
        """
        Allow a Frame to generate an ordered dict from its values

        :return: A representation of this frame object as an OrderedDict
        :rtype: OrderedDict
        """
        if self.__dict is None:
            # Create its dictionary representation
            self.__dict = OrderedDict()

            # Put the values into it
            self.__dict['_type'] = 'frame'
            self.__dict['id'] = self.__id
            self.__dict['timestamp'] = self.__timestamp
            self.__dict['error'] = self.__error
            self.__dict['protocol_stack'] = []
            self.__value_to_list(
                self.__dict['protocol_stack'],
                self.__msg.get_value()
            )
        # Return it
        return self.__dict

    @classmethod
    @typecheck
    def filter_frames(
            cls,
            frames: list_of(this_class),
            protocol: is_protocol
    ) -> (list_of(this_class), list_of(this_class)):
        """
        Allow to filter frames on a protocol

        :param frames: The frames to filter
        :param protocol:  Protocol class for filtering purposes
        :type frames: [Frame]
        :type protocol: type

        :raises TypeError: If protocol is not a protocol class
                           or if the list contains a non Frame object

        :return: A tuple containing the filtered frames and the ignored ones
        :rtype: ([Frame], [Frame])
        """

        # The return list
        filtered_frames = []
        ignored_frames = []

        # Check the protocol is one entered
        if not is_protocol(protocol):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        # Remove all frames which doesn't include this protocol
        for frame in frames:

            # If an element of the list isn't a Frame
            if not isinstance(frame, Frame):
                raise TypeError('Parameter frames contains a non Frame object')

            # If the protocol is contained into this frame
            if protocol in frame:
                filtered_frames.append(frame)
            else:
                ignored_frames.append(frame)

        # Return the newly created list
        return filtered_frames, ignored_frames

    @typecheck
    def summary(self) -> (int, str):
        """
        Allow a Frame to generate its summary

        :return: Summary of this frame
        :rtype: (int, str)
        """
        if self.__summary is None:
            one_line_msg_descriptiion = self.__msg.summary()
            # delete more that one consecutive space
            re.sub("\s\s+", " ", one_line_msg_descriptiion)
            self.__summary = (self.__id, one_line_msg_descriptiion)
        return self.__summary

    @typecheck
    def __getitem__(
            self,
            item: either(is_protocol, str)
    ) -> either(int, float, optional(Exception), str, is_layer_value):
        """
        Get the requested informations of the layer level for this frame. This
        function is also used to retrieve flat informations like src, dst, ...

        :param prot: The layer level or information that we want to retrieve
        :type prot: either(type, str)

        :return: The layer level or the information as a Value instance
        :rtype: Value

        .. seealso:: modules :ttproto:core:data:`̀MessageDescription
        """

        # If a single string, fetch the flat informations
        if isinstance(item, str):

            # If one that we get from pcap header
            if item == 'id':
                return self.__id
            elif item == 'ts':
                return self.__timestamp
            elif item == 'error':
                return self.__error
            elif item == 'value':
                return self.__msg.get_value()

            # If another one, try to get it from MessageDescription
            else:

                # Get the message description with values stored as attributes
                md = self.__msg.get_description()

                try:
                    value = getattr(md, item)
                except AssertionError:
                    raise AttributeError(
                        "%s information was not found into this frame" % item
                    ) from None  # From None suppress the first exception

                return value

        # If a protocol, fetch the layer value
        else:
            # Check that the layer is a correct protocol
            if not is_protocol(item):
                raise TypeError(item.__name__ + ' is not a protocol class')

            # Get current value
            value = self.__msg.get_value()

            # Parse the whole protocol stack
            while True:

                # If we arrive at the correct layer
                if isinstance(value, item):
                    return value

                # Go to the next layer
                try:
                    value = value['pl']
                    continue

                # If none found, leave the loop
                except (KeyError, TypeError):
                    pass
                break

            # If this protocol isn't found in the stack
            raise ProtocolNotFound(
                "%s protocol wasn't found in this frame" % item.__name__
            )


class Dissector:
    """
        Class for the dissector tool
    """

    # Class variables
    __implemented_protocols = None
    __capture = None

    @typecheck
    def __init__(self, filename: str):
        """
        The dissector tool initialisation which receives a filename

        :param filename: Filename of the pcap file to be dissected
        :type filename: str
        """

        # Get the capture of the file
        self.__capture = Capture(filename)
        logging.warning("Deprication of Dissector class and its API in favour of Capture's")

    @classmethod
    @typecheck
    def get_implemented_protocols(cls) -> list_of(type):
        """
        Allow to get the implemented protocols

        :return: Implemented protocols
        :rtype: [type]
        """
        logging.warning("Deprecation warning in favour of package's get_dissectable_protocols method!")
        # Singleton pattern
        if cls.__implemented_protocols is None:
            # Just directly get the PacketValue and InetPacketValue subclasses
            cls.__implemented_protocols = []
            # cls.__implemented_protocols += PacketValue.__subclasses__()
            # cls.__implemented_protocols += InetPacketValue.__subclasses__()

            # NOTE: This may ben needed if we change the protocol getter system
            add_subclass(cls.__implemented_protocols, PacketValue)

            # Remove the InetPacketValue class
            cls.__implemented_protocols.remove(InetPacketValue)

        # Return the singleton value
        return cls.__implemented_protocols

    @typecheck
    def summary(
            self,
            protocol: optional(is_protocol) = None
    ) -> list_of((int, str)):
        """
        The summaries function to get the summary of frames

        :param protocol: Protocol class for filtering purposes
        :type protocol: type

        :Example:

        from ttproto.core.lib.all import Ieee802154

        for s in dissector.summary(protocol = Ieee802154):

            print(s)

        :raises TypeError: If protocol is not a protocol class
        :raises ReaderError: If the reader couldn't process the file

        :return: Basic informations about frames like the underlying example
        :rtype: [(int, str)]

        :Example:

            [
                (13, '[127.0.0.1 -> 127.0.0.1] CoAP [CON 38515] GET /test'),

                (14, '[127.0.0.1 -> 127.0.0.1] CoAP [ACK 38515] 2.05 Content'),

                (21, '[127.0.0.1 -> 127.0.0.1] CoAP [CON 38516] PUT /test'),

                (22, '[127.0.0.1 -> 127.0.0.1] CoAP [ACK 38516] 2.04 Changed')]
            ]

        .. todo:: Filter uninteresting frames ? (to decrease the load)
        .. note:: With the protocol option we can filter
        """

        # Check the protocol
        if all((
                protocol,
                not is_protocol(protocol)
        )):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        # Disable the name resolution in order to improve performances
        with Data.disable_name_resolution():

            # Get the frames from the capture
            frames = self.__capture.frames

            # Filter the frames for the selected protocol
            if protocol is not None:
                frames, _ = Frame.filter_frames(frames, protocol)

        # Then give the summary of every frames
        return [frame.summary() for frame in frames]

    @typecheck
    def dissect(
            self,
            protocol: optional(is_protocol) = None
    ) -> list_of(OrderedDict):
        """
        The dissect function to dissect a pcap file into list of frames

        :param protocol: Protocol class for filtering purposes
        :type protocol: type
        :raises TypeError: If protocol is not a protocol class
        :raises ReaderError: If the reader couldn't process the file

        :return: A list of Frame represented as API's dict form
        :rtype: [OrderedDict]
        """
        # log.debug('Starting dissection.')
        # Check the protocol is one entered
        if all((
                protocol,
                not is_protocol(protocol)
        )):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        # For speeding up the process
        with Data.disable_name_resolution():

            # Get the list of frames
            frames = self.__capture.frames

            # Filter the frames for the selected protocol
            if protocol is not None:
                frames, _ = Frame.filter_frames(frames, protocol)

        # Then return the list of dictionary frame representation
        return [frame.dict() for frame in frames]


class Capture:
    """
    Class representing a Capture got from a file.

    It will give the following attributes to the users:
        - filename  => Name of the file from which the Capture was generated
        - frames  => The frame list generated
        - malformed  => The malformed frames that we didn't manage to decode

    .. note::
        The Capture object has a dictionnary of Readers in function of their
        extension
    """

    reader_extension = {
        '.pcap': PcapReader,
        '.dump': PcapReader,
        # 'json': JsonReader  # NOTE: An idea for later
    }

    @typecheck
    def __init__(self, filename: str):
        """
        Initialize a capture from .pcap filename

        :param filename: The .pcap file of the network traces capture
        :type filename: str
        """
        self._filename = filename
        self._frames = None
        self._malformed = None

        # dissect pcap capture
        self.__process_file()

    @property
    def filename(self):
        return self._filename

    @property
    def frames(self):
        if not self._frames:
            self.__process_file()
        return self._frames

    @property
    def malformed(self):
        if not self._malformed:
            self.__process_file()
        return self._malformed

    def __process_file(self):
        """
        The Capture function to decode the file into a list of frames

        :raises ReaderError: If the file was not found or if no reader matched

        .. note:: Here, we will get the reader in function of the extension
        """

        # Get the reader in function of the extension
        name, extension = path.splitext(self._filename)
        try:
            reader = self.reader_extension[extension]
        except KeyError:
            raise ReaderError(
                'No reader could be matched with %s extension' % extension
            )

        # Get an iterable reader for generating frames
        try:
            iterable_reader = reader(self._filename)
        except IOError as e:
            print("PCAP file not found. You sure %s exists? \n" % self._filename)
            raise e
        except Exception as e:
            raise ReaderError(
                "The reader wans't able to generate the frames \n" + str(e)
            ) from e  # Raise this exception from the

        # Initialize the list attributes
        self._frames = []
        self._malformed = []

        # Iterate over those tuples to generate the frames
        for count, ternary_tuple in enumerate(iterable_reader, 1):

            # The format of ternary tuple is the following:
            #   - Timestamp represented as a float
            #   - The Message object associated to the frame
            #   - An Exception if one occured, None if everything went fine

            # If not malformed (ie no exception)
            if not ternary_tuple[2]:
                self._frames.append(Frame(count, ternary_tuple))

            # If malformed
            else:
                self._frames.append(Frame(count, ternary_tuple))

    @typecheck
    def get_dissection(
            self,
            protocol: optional(is_protocol) = None,
    ) -> list_of(OrderedDict):
        """
        Function to get dissection of a capture as a list of frames represented as strings

        :param protocol: Protocol class for filtering purposes
        :type protocol: type
        :raises TypeError: If protocol is not a protocol class
        :raises ReaderError: If the reader couldn't process the file

        :return: A list of Frame represented as API's dict form
        :rtype: [OrderedDict]

        """
        # log.debug('Starting dissection.')
        # Check the protocol is one entered
        if all((
                protocol,
                not is_protocol(protocol)
        )):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        fs = self.frames

        # For speeding up the process
        with Data.disable_name_resolution():

            # Filter the frames for the selected protocol
            if protocol:
                fs, _ = Frame.filter_frames(fs, protocol)

        if fs is None:
            raise Error('Empty capture cannot be dissected')

        # Then return the list of dictionary frame representation
        return [frame.dict() for frame in fs]

    @typecheck
    def get_dissection_simple_format(
            self,
            protocol: optional(is_protocol) = None,
    ) -> list_of(str):
        """
        Function to get dissection of a capture as a list of frames represented as strings

        :param protocol: Protocol class for filtering purposes
        :type protocol: type
        :raises TypeError: If protocol is not a protocol class
        :raises ReaderError: If the reader couldn't process the file

        :return: A list of Frame represented as plain non-structured text
        :rtype: [str]

        """
        # log.debug('Starting dissection.')
        # Check the protocol is one entered
        if all((
                protocol,
                not is_protocol(protocol)
        )):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        fs = self.frames

        # For speeding up the process
        with Data.disable_name_resolution():

            # Filter the frames for the selected protocol
            if protocol:
                fs, _ = Frame.filter_frames(fs, protocol)

        if fs is None:
            raise Error('Empty capture cannot be dissected')

        # fixme modify Message class from ttproto.data structure so I can get text display wihtout this patch
        class WritableObj(object):
            def __init__(self, text=''):
                self.val = text

            def __str__(self):
                return self.val

            def write(self, text):
                self.val += text

        frame_dissection_list = []

        for f in fs:
            text_output = WritableObj()
            f.message.display(
                output=text_output
            )
            frame_dissection_list.append(str(text_output))

        # Then return the list of frames,each as a simple text dissection
        return frame_dissection_list

    @typecheck
    def summary(
            self,
            protocol: optional(is_protocol) = None
    ) -> list_of((int, str)):
        """
        The summaries function to get the summary of frames

        :param protocol: Protocol class for filtering purposes
        :type protocol: type

        :Example:

        from ttproto.core.lib.all import Ieee802154

        for s in dissector.summary(protocol = Ieee802154):

            print(s)

        :raises TypeError: If protocol is not a protocol class
        :raises ReaderError: If the reader couldn't process the file

        :return: Basic informations about frames like the underlying example
        :rtype: [(int, str)]

        :Example:

            [
                (13, '[127.0.0.1 -> 127.0.0.1] CoAP [CON 38515] GET /test'),

                (14, '[127.0.0.1 -> 127.0.0.1] CoAP [ACK 38515] 2.05 Content'),

                (21, '[127.0.0.1 -> 127.0.0.1] CoAP [CON 38516] PUT /test'),

                (22, '[127.0.0.1 -> 127.0.0.1] CoAP [ACK 38516] 2.04 Changed')]
            ]

        .. note:: With the protocol option we can filter the response
        """

        if all((
                protocol,
                not is_protocol(protocol)
        )):
            raise TypeError(protocol.__name__ + ' is not a protocol class')

        fs = self.frames

        # For speeding up the process
        with Data.disable_name_resolution():

            # Filter the frames for the selected protocol
            if protocol:
                fs, _ = Frame.filter_frames(fs, protocol)

        if fs is None:
            raise Error('Empty capture cannot be dissected')

        # Return list of frames summary
        return [frame.summary() for frame in fs]


if __name__ == "__main__":
    import json
    import logging

    cap = Capture(
        #    'tests/test_dumps/DissectorTests/coap/CoAP_plus_random_UDP_messages.pcap'
        'tests/test_dumps/coap/CoAP_plus_random_UDP_messages.pcap'
        # 'tmp/frame2_onem2m.pcap'
        # 'tmp/frame1_onem2m.pcap'
    )
    dis = cap.get_dissection_simple_format()
    # for i in dis:
    #     print(i)
    #     print('\n' * 2)
    #
    # print('********' * 2)

    # print(cap.summary())
    # print('#####')
    # print(cap.summary())
    # print('##### Dissect with filtering on CoAP #####')
    # print(cap.get_dissection(CoAP))
    # print('#####')
    # print('##### Dissect without filtering #####')
    # print(json.dumps(cap.get_dissection(), indent=4))
    # print('#####')
    # print('#####')
    # print(Dissector.get_implemented_protocols())
    # capture = Capture('/'.join((
    #     'tests',
    #     'test_dumps',
    #     'TD_COAP_CORE_02_MULTIPLETIMES.pcap'
    # )))
    # try:
    #     capt = Capture('/'.join((
    #         'tests',
    #         'test_dumps',
    #         'NON_EXISTENT.pcap'
    #     )))
    #     capt.frames
    # except ReaderError:
    #     print('File not found correctly managed')
    # try:
    #     capt = Capture('/'.join((
    #         'tests',
    #         'test_files',
    #         'WrongFilesForTests',
    #         'not_a_pcap_file.dia'
    #     )))
    #     capt.frames
    # except ReaderError:
    #     print('Reader not found correctly managed')
    # try:
    #     capt = Capture('/'.join((
    #         'tests',
    #         'test_files',
    #         'WrongFilesForTests',
    #         'empty_pcap.pcap'
    #     )))
    #     capt.frames
    # except ReaderError:
    #     print('Reader error correctly managed (empty file)')
    # try:
    #     capture.frames = []
    # except AttributeError:
    #     print('Writting capture frames correctly blocked')
    # try:
    #     capture.malformed = []
    # except AttributeError:
    #     print('Writting capture malformed frames correctly blocked')
    # try:
    #     capture.filename = ''
    # except AttributeError:
    #     print('Writting capture filename correctly blocked')
    # print('##### Frames')
    # print(capture.frames)
    # print('##### Malformed')
    # print(capture.malformed)
    # print('##### Second time Frames')
    # print(capture.frames)
    # print('##### Second time Malformed')
    # print(capture.malformed)
    # frame_list = Capture(
    #     '/'.join((
    #         'tests',
    #         'test_dumps',
    #         'TD_COAP_CORE_07_FAIL_No_CoAPOptionContentFormat_plus_random_UDP_messages.pcap'
    #     ))
    # ).frames
    # frame_list, _ = Frame.filter_frames(frame_list, CoAP)
    # print(frame_list[0]['value'])
    # print(frame_list[0][IPv4])
    # print(frame_list[0]['ts'])
    # print(frame_list[0]['value'])
    # print(frame_list[0][CoAP])
    # print(frame_list[0][CoAP]['Type'])
    # try:
    #     print(frame_list[0][Value])
    # except InputParameterError:
    #     pass
    # try:
    #     print(frame_list[0][IPv6])
    # except ProtocolNotFound as e:
    #     print(e)
    # for f in frame_list:
    #     if CoAP in f:
    #         print(f[CoAP])
    #         print(f[CoAP]['type'])
    #         print(f[CoAP]['pl'])
    #         print(f['id'])
    #         print(f['ts'])
    #         print(f['error'])
    #         print(f['src'])
    #         print(f['dst'])
    #         print(f['hw_src'])
    #         print(f['hw_dst'])
    #         print(f['src_port'])
    #         print(f['dst_port'])
    # for frame in frame_list:
    #     try:
    #         print(frame[CoAP]['opt'][CoAPOptionMaxAge]['val'])
    #     except KeyError:
    #         pass
    # try:
    #     print(frame_list[0]['Unknown'])
    # except AttributeError:
    #     print('Fetching an unknown attribute correclty throw an error')
    # frame_list = Capture(
    #     '/'.join((
    #         'tests',
    #         'test_dumps',
    #         'wireshark_official_6lowpan_sample.pcap'
    #     ))
    # ).frames
    # frame_list = Capture(
    #     '/'.join((
    #         'tests',
    #         'test_dumps',
    #         'www.cloudshark.org_captures_46a9a369e6a9.pcap'
    #     ))
    # ).frames
    # frame_list, ignored = Frame.filter_frames(frame_list, Ethernet)
    # print('The frame list contains %d elements:' % len(frame_list))
    # c = 0
    # for f in frame_list:
    #     print('%d: %s' % (c, f['value']))
    #     c += 1
    # c = 0
    # for i in ignored:
    #     print('%d: %s' % (c, i['value']))
    #     c += 1
    # pass



from ..common import *


class TD_IPSO_3306_01(CoAPTestCase):
    """
testcase_id: TD_IPSO_3306_01
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: 
  - Querying Actuatuation Object instance (ID = 3306) representation in JSON format on LwM2M client
pre_conditions: Device is registered at the LWM2M server
sequence:
  - step_id: 'TD_IPSO_3306_01_step_01'
    type: stimuli
    node : lwm2m_server
    description:
      - 'Server sends a READ request (COAP GET) on Actuation Object'
      - - Type = 0 (CON)
        - Code = 1 (GET)

  - step_id: 'TD_IPSO_3306_01_step_02'
    type: check
    description:
      - 'The request sent by the server contains'
      - - Type=0 and Code=1
        - Accept option = application/vnd.oma.lwm2m+json
        - Uri-Path option = 3306/0

  - step_id: 'TD_IPSO_3306_01_step_03'
    type: check
    description:
        - 'Client sends response containing'
        - - Code = 2.05 (Content)
          - content-format option = application/vnd.oma.lwm2m+json
          - Non-empty Payload

  - step_id: 'TD_IPSO_3306_01_step_04'
    type: verify
    node: lwm2m_server
    description:
        - 'Requested data is successfully displayed'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='get')]

    def run(self):
        self.match('server', CoAP(type='con', code='get', opt=self.uri('/3306/0')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11543'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11543'))), 'fail')

        validation = validate(str(self.coap['pl']),'3306')
        self.set_verdict(validation[0], validation[1])



     



#!/usr/bin/env python3
#
#  (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import http.server
import io
import sys
import os
import errno
import tempfile
import re
import time
import signal
import select
import subprocess
import cgi
import json
import hashlib
import base64
import requests
import email.feedparser
import email.message

from collections import OrderedDict
from urllib.parse import urlparse, parse_qs
from ttproto.utils import pure_pcapy
from ttproto.core.analyzer import Analyzer
from ttproto.core.dissector import Dissector
from ttproto.core.xmlgen import XHTML10Generator, XMLGeneratorControl
from ttproto.core.typecheck import *
from ttproto.core.lib.all import *

# Directories
from ttproto import DATADIR
from ttproto import TMPDIR
from ttproto import LOGDIR

# List to generate the changelog page
CHANGELOG = []
CHANGELOG_FIRST_COMMIT = "iot2-beta"


# ########################## ttproto API ########################### #

# Prefix and suffix for the hashes
HASH_PREFIX = 'tt'
HASH_SUFFIX = 'proto'
TOKEN_LENGTH = 28

# The different implemented protocols
PROTOCOLS = OrderedDict()


# ######################## End of API part ######################### #


def prepare_changelog():
    global CHANGELOG
    CHANGELOG = []

    try:
        lines = None

        # NOTE: limite log to first parent to avoid the burden of having
        #       verbose changelog about tool updates
        #    -> put tool updates in branch 'tool'
        #    -> put test suites updates in branch 'master'
        #    -> merge 'tool' into 'master' using « --no-ff --no-commit »
        #       and put summary changes about the tool
        git_cmd = [
            'git',
            'log',
            '--format=format:%cD\n%h\n%d\n%B\n_____end_commit_____',
            '--first-parent'
        ]

        if CHANGELOG_FIRST_COMMIT:
            git_cmd.append("%s^1.." % CHANGELOG_FIRST_COMMIT)

        git_log = iter(str(subprocess.Popen(
                git_cmd,
                stdout=subprocess.PIPE,
                close_fds=True,
            ).stdout.read(), "utf8", "replace").splitlines())

        while True:
            date = next(git_log)
            ver = next(git_log)
            tags = next(git_log)
            if tags:
                tags = tags[2:-1]
            lines = []
            while True:
                l = next(git_log)
                if l == "_____end_commit_____":
                    break
                lines.append(l)

            if lines and lines[-1] != "":
                lines.append("")  # ensure that there will be a \n at the end
            CHANGELOG.append((ver, tags, date, "\n".join(lines)))
    except StopIteration:
        pass
    except Exception as e:
        CHANGELOG = [(
            ("error when generating changelog(%s: %s)" % (type(e).__name__, e)),
            "",
            "",
            ""
        )]


def html_changelog(g):

    ctl = XMLGeneratorControl(g)

    g.h2("Changelog")
    for ver, tags, date, body in CHANGELOG:
        if tags:
            g.b("%s" % (tags))
            ctl.raw_write("<br>")  # FIXME: bug in xmlgen
        g.span("%s - %s\n\n" % (ver, date), style="color:#808080")

        g.pre("\t%s\n\n" % "\n\t".join(body.splitlines()))


# ########################## ttproto API ########################### #

@typecheck
def get_test_cases(
    testcase_id: optional(str) = None
) -> OrderedDict:
    """
    Function to get the test cases from files if not initialized

    Or directly from the global storage variable, can retrieve a single test
    case if wanted but doesn't use the analysis function, cf the remark

    :param testcase_id: The id of the single test case if one wanted
    :type testcase_id: str

    :return: The implemented test cases
    :rtype: OrderedDict
    """

    # New way by analyzer tool
    test_cases = OrderedDict()
    tc_query = [] if not testcase_id else [testcase_id]
    raw_tcs = Analyzer('tat_coap').get_implemented_testcases(tc_query)

    # Build the clean results list
    for raw_tc in raw_tcs:

        tc_basic = OrderedDict()
        tc_basic['_type'] = 'tc_basic'
        tc_basic['id'] = raw_tc[0]
        tc_basic['objective'] = raw_tc[1]

        tc_implementation = OrderedDict()
        tc_implementation['_type'] = 'tc_implementation'
        tc_implementation['implementation'] = raw_tc[2]

        # Tuple, basic + implementation
        test_cases[raw_tc[0]] = OrderedDict()
        test_cases[raw_tc[0]]['tc_basic'] = tc_basic
        test_cases[raw_tc[0]]['tc_implementation'] = tc_implementation

    # If a single element is asked
    if testcase_id:
        test_cases = test_cases[raw_tcs[0][0]]

    # Return the results
    return test_cases


@typecheck
def get_protocol(
    protocol: optional(str) = None
) -> either(OrderedDict, type(None)):
    """
    Function to get the protocols

    :param protocol: The name of the protocol
    :type protocol: str

    :return: The implemented protocols, or a single one
    :rtype: OrderedDict
    """

    # Get the global variable to store the protocols
    global PROTOCOLS

    # If empty for the moment
    if len(PROTOCOLS) == 0:

        # Put the 'None' protocol which get everything
        PROTOCOLS['None'] = OrderedDict()
        PROTOCOLS['None']['_type'] = 'implemented_protocol'
        PROTOCOLS['None']['name'] = 'None'
        PROTOCOLS['None']['description'] = ''

        # Getter of protocol's classes from dissector
        prot_classes = Dissector.get_implemented_protocols()

        # Build the clean results list
        for prot_class in prot_classes:

            # Put the protocol
            prot = OrderedDict()
            prot['_type'] = 'implemented_protocol'
            prot['name'] = prot_class.__name__
            prot['description'] = ''

            # Add it to protocol variable
            PROTOCOLS[prot['name']] = prot

    # If there's only one protocol asked
    if protocol is not None:
        if protocol in PROTOCOLS:
            return PROTOCOLS[protocol]
        else:
            return None

    # Rerurn the protocols
    else:
        return PROTOCOLS


@typecheck
def correct_get_param(par: list, is_number: optional(bool) = False) -> bool:
    """
    Function to check if a get parameter is correct

    :param par: The get parameter to check (care it's a list)
    :param is_number: If we expect it to be a number
    :type par: list
    :type is_number: bool

    :return: True if the get parameter is correct, False if not
    :rtype: bool
    """
    return all((
        len(par) == 1,
        type(par[0]) == str,
        par[0] != '',
        not (
            is_number and (
                not par[0].isdigit() or int(par[0]) <= 0
            )
        )
    ))


@typecheck
def get_token(tok: optional(str) = None):
    """
    Function to get a token, if there's a valid one entered just return it
    otherwise generate a new one

    :param tok: The token if there's already one
    :type tok: str

    :return: A token, the same if there's already one, a new one otherwise
    :rtype: str
    """

    # If the token is already a correct one
    try:
        if all((
            tok,
            type(tok) == str,
            len(tok) == 28,
            base64.urlsafe_b64decode(tok + '=')  # Add '=' only for checking
        )):
            return tok
    except:  # If the decode throw an error => Wrong base64
        pass

    # Generate a token because there is none
    token = hashlib.sha1(
        str.encode((
            "%s%s%04d%s" %
            (
                HASH_PREFIX,
                time.time(),
                job_id,
                HASH_SUFFIX
            )
        ), encoding='utf-8')
    )
    token = base64.urlsafe_b64encode(token.digest()).decode()

    # Remove the '=' at the end of it, it is used by base64 for padding
    return token.replace('=', '')


# ######################## End of API part ######################### #


class UTF8Wrapper(io.TextIOBase):
    def __init__(self, raw_stream):
        self.__raw = raw_stream

    def write(self, string):

        return self.__raw.write(bytes(string, "utf-8"))


class BytesFeedParser(email.feedparser.FeedParser):
    """Like FeedParser, but feed accepts bytes."""

    def feed(self, data):
        super().feed(data.decode('ascii', 'surrogateescape'))


class RequestHandler(http.server.BaseHTTPRequestHandler):

    # ########################## ttproto API ########################### #
    def api_error(self, message):
        """
            Function for generating a json error
            The error message is logged at the same time
        """
        self.log_message("%s error: %s", self.path, message)
        to_dump = OrderedDict()
        to_dump['_type'] = 'response'
        to_dump['ok'] = False
        to_dump['error'] = message
        print(json.dumps(to_dump))

    # ######################## End of API part ######################### #

    def log_message(self, format, *args, append=""):
        global log_file
        host = self.address_string()
        if host in("172.17.42.1", "localhost", "127.0.0.1", "::1"):
            xff = self.headers.get("x-forwarded-for")
            if xff:
                host = xff

        txt = ("%s - - [%s] %s - %s\n%s" %
               (host,
                self.log_date_time_string(),
                format % args,
                self.headers.get("user-agent"),
                "".join("\t%s\n" % l for l in append.splitlines()),
                ))

        sys.stderr.write(txt)
        log_file.write(txt)
        log_file.flush()

    def do_GET(self):

        # Get the url and parse it
        url = urlparse(self.path)

        if url.path == "/coap-tool.sh":
            fp = open("coap-tool.sh", "rb")

            if not fp:
                self.send_response(500)
                return

            self.send_response(200)
            self.send_header("Content-Type", "text/x-sh")
            self.end_headers()

            self.wfile.write(fp.read())
            return
        elif url.path == "/doc/ETSI-CoAP4-test-list.pdf":
            fp = open("doc/ETSI-CoAP4-test-list.pdf", "rb")

            if not fp:
                self.send_response(500)
                return

            self.send_response(200)
            self.send_header("Content-Type", "application/pdf")
            self.end_headers()

            self.wfile.write(fp.read())
            return
        elif url.path == "/doc/Additive-IRISA-CoAP-test-list.pdf":
            fp = open("doc/Additive-IRISA-CoAP-test-list.pdf", "rb")

            if not fp:
                self.send_response(500)
                return

            self.send_response(200)
            self.send_header("Content-Type", "application/pdf")
            self.end_headers()

            self.wfile.write(fp.read())
            return
        elif url.path == "/doc/Additive-IRISA-CoAP-test-description.pdf":
            fp = open("doc/Additive-IRISA-CoAP-test-description.pdf", "rb")

            if not fp:
                self.send_response(500)
                return

            self.send_response(200)
            self.send_header("Content-Type", "application/pdf")
            self.end_headers()

            self.wfile.write(fp.read())
            return

        # ########################## ttproto API ########################### #

        # ##### Personnal remarks
        #
        # For the moment, using this webserver is right but for scaling maybe a
        # strong web platform using a framework will be better. This one is
        # sufficient for the moment.
        #
        # We check on the path for whole uri, maybe we should bind a version to
        # a beginning like "/api/v1" and then bind the methods put behind it.
        #
        # ##### End of remarks

        # GET handler for the analyzer_getTestCases uri
        # It will give to the gui the list of the test cases
        #
        elif url.path == '/api/v1/analyzer_getTestCases':

            # Send the header
            self.send_response(200)
            self.send_header("Content-Type", "application/json;charset=utf-8")
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the list of test cases
            try:
                test_cases = get_test_cases()
            except FileNotFoundError as fnfe:
                self.api_error(
                    'Problem during fetching the test cases list:\n' + str(fnfe)
                )
                return

            clean_test_cases = []
            for tc in test_cases:
                clean_test_cases.append(test_cases[tc]['tc_basic'])

            # If no test case found
            if len(clean_test_cases) == 0:
                self.api_error('No test cases found')
                return

            # The result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True
            json_result['content'] = clean_test_cases

            # Just give the json representation of the test cases list
            print(json.dumps(json_result))
            return

        # GET handler for the analyzer_getTestcaseImplementation uri
        # It will allow developpers to get the implementation script of a TC
        #
        # /param testcase_id => The unique id of the test case
        #
        elif url.path == '/api/v1/analyzer_getTestcaseImplementation':

            # Send the header
            self.send_response(200)
            self.send_header("Content-Type", "application/json;charset=utf-8")
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the parameters
            params = parse_qs(url.query)

            try:

                # Check parameters
                if any((
                    len(params) != 1,
                    'testcase_id' not in params,
                    not correct_get_param(params['testcase_id'])
                )):
                    raise

            # Catch errors (key mostly) or if wrong parameter
            except:
                self.api_error(
                    "Incorrects GET parameters, expected '?testcase_id={string}'"
                )
                return

            # Get the test case
            try:
                test_case = get_test_cases(params['testcase_id'][0])
            except FileNotFoundError:
                self.api_error(
                    'Test case %s not found' % params['testcase_id'][0]
                )
                return

            # The result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True
            json_result['content'] = [
                test_case['tc_basic'],
                test_case['tc_implementation']
            ]

            # Here the process from ttproto core
            print(json.dumps(json_result))
            return

        # GET handler for the analyzer_getProtocols uri
        # It will give to the gui the list of the protocols implemented
        #
        elif url.path in (
            '/api/v1/analyzer_getProtocols',
            '/api/v1/dissector_getProtocols'
        ):

            # Send the header
            self.send_response(200)
            self.send_header("Content-Type", "application/json;charset=utf-8")
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # The result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True
            json_result['content'] = []

            # Get the protocols
            protocols = get_protocol()
            for prot in protocols:
                json_result['content'].append(protocols[prot])

            # Just give the json representation of the test cases list
            print(json.dumps(json_result))
            return

        # GET handler for the analyzer_getFrames and dissector_getFrames uri
        # It will allow an user to get a single frame from the previous pcap
        # if no frame_id provided, otherwise it will return all the frames
        #
        # /param token => The token of the corresponding pcap file
        # /param protocol_selection => The protocol we want to filter on
        # /param frame_id (optional) => The id of the wanted frame
        #
        # /remark We redirect to the same function but later those two
        # may diverge
        #
        elif url.path in (
            '/api/v1/analyzer_getFrames',
            '/api/v1/dissector_getFrames'
        ):

            # Send the header
            self.send_response(200)
            self.send_header("Content-Type", "application/json;charset=utf-8")
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the parameters
            params = parse_qs(url.query)
            try:

                # Only token and protocol_selection
                if len(params) == 2:
                    if any((
                        'token' not in params,
                        'protocol_selection' not in params,
                        not correct_get_param(params['token']),
                        not correct_get_param(params['protocol_selection'])
                    )):
                        raise

                # token, protocol_selection and frame_id
                elif len(params) == 3:
                    if any((
                        'token' not in params,
                        'protocol_selection' not in params,
                        'frame_id' not in params,
                        not correct_get_param(params['token']),
                        not correct_get_param(params['frame_id'], is_number=True),
                        not correct_get_param(params['protocol_selection'])
                    )):
                        raise

                # Wrong number of parameters
                else:
                    raise

            # Catch errors (key mostly) or if wrong parameter
            except:
                self.api_error(
                    "Incorrects parameters expected '?token={string}&protocol_selection={string}(&frame_id={integer})?'"
                )
                return

            # Format the id before passing it to the process function
            token = params['token'][0]
            protocol_selection = params['protocol_selection'][0]

            # Check the protocol
            protocol = get_protocol(protocol_selection)
            if protocol is None:
                self.api_error(
                    'Unknown %s protocol' % protocol_selection
                )
                return

            # Create the frames object
            frames = []

            # Get the json file
            try:
                json_path = os.path.join(
                    TMPDIR,
                    "%s.json" % token
                )
                with open(json_path, 'r') as json_fp:
                    frames = json.load(json_fp, object_pairs_hook=OrderedDict)
            except:
                self.api_error(
                    'Session identified by token %s not found' % token
                )
                return

            # If a single file asked
            if len(params) == 3:
                frame_id = int(params['frame_id'][0])
                for frame in frames:
                    if frame['id'] == frame_id:
                        frames = [frame]

                # If no frame with this id
                if len(frames) != 1:
                    self.api_error(
                        'No frame with id=%u found' % frame_id
                    )
                    return

            # The result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True

            token_res = OrderedDict()
            token_res['_type'] = 'token'
            token_res['value'] = token

            frames.insert(0, token_res)

            json_result['content'] = frames

            # Dump the json result
            print(json.dumps(json_result))
            return

        # GET handler for the dissector_getFramesSummary uri
        # It will allow an user to get all the frames summary
        #
        # /param token => The token of the corresponding pcap file
        # /param protocol_selection => The protocol we want to filter on
        #
        elif url.path == '/api/v1/dissector_getFramesSummary':

            # Send the header
            self.send_response(200)
            self.send_header("Content-Type", "application/json;charset=utf-8")
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the parameters
            params = parse_qs(url.query)
            try:

                # Only token and protocol_selection
                if any((
                    len(params) != 2,
                    'token' not in params,
                    'protocol_selection' not in params,
                    not correct_get_param(params['token']),
                    not correct_get_param(params['protocol_selection'])
                )):
                    raise

            # Catch errors (key mostly) or if wrong parameter
            except:
                self.api_error(
                    "Incorrects parameters expected '?token={string}&protocol_selection={string}'"
                )
                return

            # Format the id before passing it to the process function
            token = params['token'][0]
            protocol_selection = params['protocol_selection'][0]

            # Check the protocol
            protocol = get_protocol(protocol_selection)
            if protocol is None:
                self.api_error(
                    'Unknown %s protocol' % protocol_selection
                )
                return

            # Get the dump file
            pcap_path = os.path.join(
                TMPDIR,
                "%s.dump" % token
            )
            try:
                # Get summaries from it
                frames_summary = Dissector(pcap_path).summary(
                    eval(protocol['name'])
                )
            except TypeError as e:
                self.api_error('Dissector error:\n' + str(e))
                return
            except:
                self.api_error(
                    'Session identified by token %s not found' % token
                )
                return

            # The result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True

            token_res = OrderedDict()
            token_res['_type'] = 'token'
            token_res['value'] = token
            json_result['content'] = []
            json_result['content'].append(token_res)

            # Add each frame summary
            for f_id, f_sum in frames_summary:
                summary = OrderedDict()
                summary['_type'] = 'frame_summary'
                summary['frame_id'] = f_id
                summary['frame_summary'] = f_sum
                json_result['content'].append(summary)

            # Dump the json result
            print(json.dumps(json_result))
            return

        # ######################## End of API part ######################### #

        elif url.path != "/":
            self.send_error(404)
            return

        self.send_response(200)
        self.send_header("Content-Type", "text/html;charset=utf-8")
        self.end_headers()

        with XHTML10Generator(output=UTF8Wrapper(self.wfile)) as g:

            with g.head:
                g.title("IRISA CoAP interoperability Testing Tool")
                g.style("img {border-style: solid; border-width: 20px; border-color: white;}", type="text/css")

            g.h1("IRISA CoAP interoperability Testing Tool")

            g.b("Tool version: ")
            # g("%s" % Analyzer.TOOL_VERSION)
            with g.br():  # FIXME: bug generator
                pass
            with g.form(method="POST", action="submit", enctype="multipart/form-data"):
                g("This tool(more details at ")
                g.a("www.irisa.fr/tipi", href="http://www.irisa.fr/tipi/wiki/doku.php/passive_validation_tool_for_coap")
                g(") allows executing CoAP interoperability test suites(see below Available Test Scenarios) on the provided traces of CoAP Client-Server interactions.")
                g.br()
                g.h3("Available Test Scenarios:")
                g("- ETSI COAP#4 Plugtest scenarios: ")
                g.a("ETSI-CoAP4-test-list", href="doc/ETSI-CoAP4-test-list.pdf")
                g(", ")
                g.a("ETSI-CoAP4-test-description", href="https://github.com/cabo/td-coap4/")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("- Additive test scenarios developed by IRISA/Tipi Group: ")
                g.a("Additive-IRISA-CoAP-test-list", href="doc/Additive-IRISA-CoAP-test-list.pdf")
                g(", ")
                g.a("Additive-IRISA-CoAP-test-description", href="doc/Additive-IRISA-CoAP-test-description.pdf")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g.h3("IETF RFCs/Drafts covered:")
                g("- CoAP CORE(")
                g.a("RFC7252", href="http://tools.ietf.org/html/rfc7252")
                g(")")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("- CoAP OBSERVE(")
                g.a("draft-ietf-core-observe-16", href="http://tools.ietf.org/html/draft-ietf-core-observe-16")
                g(")")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("- CoAP BLOCK(")
                g.a("draft-ietf-core-block-17", href="http://tools.ietf.org/html/draft-ietf-core-block-17")
                g(")")
                g.br()
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass

                g("==========================================================================================")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("Submit your traces(pcap format). \nWarning!! pcapng format is not supported; you should convert your pcapng file to pcap format.")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g.input(name="file", type="file", size=60)
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("Configuration")
                g.br()
                with g.select(name="profile"):
                    g.option("Client <-> Server", value="client", selected="1")
                    g.option("Reverse-Proxy <-> Server", value="reverse-proxy")
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                g("Optional regular expression for selecting scenarios(eg: ")
                g.tt("CORE_0[1-2]")
                g(" will run only ")
                g.tt("TD_COAP_CORE_01")
                g(" and ")
                g.tt("TD_COAP_CORE_02")
                g(")")
                g.br()
                g.input(name="regex", size=60)
                g.br()
                with g.br():  # FIXME: bug in generation if we remove the with context
                    pass
                with g.input(name="agree", type="checkbox", value="1"):
                    pass
                g("I agree to leave a copy of this file on the server(for debugging purpose). Thanks")

                g.br()

                with g.input(name="urifilter", type="checkbox", value="1"):
                    pass
                g("Filter conversations by URI(/test vs. /separate vs. /.well-known/core  ...) to reduce verbosity")

                g.br()
                g.br()
                g.input(type="submit")
                with g.br():  # FIXME: bug generator
                    pass
                g.b("Note:")
                g(" alternatively you can use the shell script ")
                g.a("coap-tool.sh", href="coap-tool.sh")
                g(" to capture and submit your traces to the server(requires tcpdump and curl installed on your system).")

            g.a(href="http://www.irisa.fr/tipi").img(src="http://www.irisa.fr/tipi/wiki/lib/tpl/tipi_style/images/irisa.jpg", height="40")

            g.a(href="http://www.irisa.fr/tipi").img(src="http://www.irisa.fr/tipi/wiki/lib/tpl/tipi_style/images/tipi_small.png", height="50")

            html_changelog(g)

    def do_POST(self):

        # The job counter
        global job_id
        job_id += 1

        # ########################## ttproto API ########################### #

        # POST handler for the analyzer_testCaseAnalyze uri
        # It will allow users to analyze a pcap file corresponding to a TC
        #
        # \param pcap_file => The pcap file that we want to analyze
        # \param token => The token previously provided
        # \param testcase_id => The id of the corresponding test case
        # The pcap_file or the token is required, having both is also forbidden
        #
        if self.path == '/api/v1/analyzer_testCaseAnalyze':

            # Send the header
            self.send_response(200)
            self.send_header('Content-Type', 'application/json;charset=utf-8')
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the content type
            try:
                content_type = cgi.parse_header(self.headers['Content-Type'])
            except TypeError:
                self.api_error(
                    "Non empty POST datas and format of 'multipart/form-data' expected"
                )
                return

            # Get post values
            form = cgi.FieldStorage(
                fp=self.rfile,
                headers=self.headers,
                keep_blank_values=True,
                environ={
                    'REQUEST_METHOD': 'POST',
                    'CONTENT_TYPE': content_type[0]
                })

            # Check that we have the two values
            if any((
                len(form) != 2,
                'testcase_id' not in form,
                all((  # None of the two required => Error
                    'pcap_file' not in form,
                    'token' not in form
                )),
                all((  # Both of them => Error
                    'pcap_file' in form,
                    'token' in form
                ))
            )):
                self.api_error(
                    'Expected POST=([pcap_file={file}|token={text}], testcase_id={text})'
                )
                return

            # Get the test case and its informations
            testcase_id = form.getvalue('testcase_id')
            if not type(testcase_id) == str:
                self.api_error('The value of the testcase_id should be a string from text input')
                return

            # Try to get the test case
            try:
                test_case = get_test_cases(testcase_id)
            except FileNotFoundError:
                self.api_error('Test case %s not found' % testcase_id)
                return

            # Get the token
            token = form.getvalue('token')

            # Get analysis results from the token
            if token:

                # Just get the path
                pcap_path = os.path.join(
                    TMPDIR,
                    token + '.dump'
                )

            # Get analysis results from the pcap file
            else:

                # Check headers
                if any((
                    len(content_type) == 0,
                    content_type[0] is None,
                    content_type[0] != 'multipart/form-data'
                )):
                    self.api_error(
                        "POST format of 'multipart/form-data' expected, no file input 'pcap_file' found"
                    )
                    return

                # Get the same token or generate a new one
                token = get_token(token)

                # Get and check the pcap file entered
                pcap_file = form.getvalue('pcap_file')

                # Path to save the file
                pcap_path = os.path.join(
                    TMPDIR,
                    token + '.dump'
                )

                # Write the pcap file to a temporary destination
                try:
                    with open(pcap_path, 'wb') as f:
                        f.write(pcap_file)
                except:
                    self.api_error(
                        "Couldn't write the temporary file %s"
                        %
                        pcap_path
                    )
                    return

                # Get the dissection from analysis tool
                try:
                    dissection = Dissector(pcap_path).dissect()
                except pure_pcapy.PcapError:
                    self.api_error(
                        "Expected 'pcap_file' to be a non empty pcap file"
                    )
                except:
                    self.api_error(
                        "Couldn't read the temporary file %s"
                        %
                        pcap_path
                    )
                    return

                # Save the json dissection result into a file
                json_save = os.path.join(
                    TMPDIR,
                    token + '.json'
                )
                try:
                    with open(json_save, 'w') as f:
                        json.dump(dissection, f)
                except:
                    self.api_error("Couldn't write the json file")
                    return

            # Get the result of the analysis
            analysis_results = Analyzer('tat_coap').analyse(
                                pcap_path,
                                testcase_id
                            )

            # self.log_message("###############################################")
            # self.log_message("Verdict description is : %s", analysis_results[0][3])
            # self.log_message("###############################################")

            # print(analysis_results)

            # Error for some test cases that the analysis doesn't manage to get
            try:
                assert type(analysis_results) == tuple
                assert len(analysis_results) == 5
                assert type(analysis_results[0]) == str
                assert type(analysis_results[1]) == str
                assert type(analysis_results[2]) == list
                assert type(analysis_results[3]) == str
                assert type(analysis_results[5]) == list
                for exception_tuple in analysis_results[5]:
                    assert type(exception_tuple) == tuple
                    assert len(exception_tuple) == 3
                    assert isinstance(exception_tuple[0], type)
                    assert isinstance(exception_tuple[1], Exception)
                    assert isinstance(exception_tuple[2], object)
                assert analysis_results[0] == test_case['tc_basic']['id']
            except AssertionError:
                self.api_error(
                    'Problem with the analyse of TC %s, wrong result received'
                    %
                    testcase_id
                )
                return

            # Only take the first
            verdict = OrderedDict()
            verdict['_type'] = 'verdict'
            verdict['verdict'] = analysis_results[1]
            verdict['description'] = analysis_results[3]
            verdict['review_frames'] = analysis_results[2]

            token_res = OrderedDict()
            token_res['_type'] = 'token'
            token_res['value'] = token

            # Prepare the result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True
            json_result['content'] = [
                token_res,
                test_case['tc_basic'],
                verdict
            ]

            # Here we will analyze the pcap file and get the results as json
            print(json.dumps(json_result))
            return

        # POST handler for the analyzer_allMightyAnalyze uri
        # It will allow users to analyze a pcap file without giving
        # a corresponding test case
        #
        # \param pcap_file => The pcap file that we want to analyze
        # \param token => The token previously provided
        # The pcap_file or the token is required, having both is also forbidden
        #
        elif self.path == '/api/v1/analyzer_allMightyAnalyze':

            # Send the header
            self.send_response(200)
            self.send_header('Content-Type', 'application/json;charset=utf-8')
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Not implemented for the moment
            self.api_error(
                "This method is not implemented yet, please come back later"
            )
            return

        # POST handler for the dissector_dissectFile uri
        # It will allow users to analyze a pcap file corresponding to a TC
        #
        # \param pcap_file => The pcap file that we want to dissect
        # \param protocol_selection => The protocol name
        #
        elif self.path == '/api/v1/dissector_dissectFile':

            # Send the header
            self.send_response(200)
            self.send_header('Content-Type', 'application/json;charset=utf-8')
            self.end_headers()

            # Bind the stdout to the http output
            os.dup2(self.wfile.fileno(), sys.stdout.fileno())

            # Get the content type
            try:
                content_type = cgi.parse_header(self.headers['Content-Type'])
            except TypeError:
                self.api_error(
                    "Non empty POST datas and format of 'multipart/form-data' expected"
                )
                return

            # Check headers
            if any((
                len(content_type) == 0,
                content_type[0] is None,
                content_type[0] != 'multipart/form-data'
            )):
                self.api_error(
                    "POST format of 'multipart/form-data' expected, no file input 'pcap_file' found"
                )
                return

            # Get post values
            form = cgi.FieldStorage(
                fp=self.rfile,
                headers=self.headers,
                environ={
                    'REQUEST_METHOD': 'POST',
                    'CONTENT_TYPE': content_type[0]
                })

            # Check the parameters passed
            if any((
                len(form) != 2,
                'pcap_file' not in form,
                'protocol_selection' not in form
            )):
                self.api_error(
                    'Expected POST=(pcap_file={file}, protocol_selection={text})'
                )
                return

            # Check the protocol_selection value
            protocol_selection = form.getvalue('protocol_selection')
            if not type(protocol_selection) == str:
                self.api_error('Expected protocol_selection post value to be a text (eq string)')
                return

            # In function of the protocol asked
            prot = get_protocol(protocol_selection)
            if prot is None:
                self.api_error('Unknown protocol %s' % protocol_selection)
                return

            # Generate a new token
            token = get_token()

            # Get the pcap file
            pcap_file = form.getvalue('pcap_file')

            # Path to save the file
            pcap_path = os.path.join(
                TMPDIR,
                token + '.dump'
            )

            # Write the pcap file to a temporary destination
            try:
                with open(pcap_path, 'wb') as f:
                    f.write(pcap_file)
            except:
                self.api_error("Couldn't write the temporary file")
                return

            # Prepare the result to return
            json_result = OrderedDict()
            json_result['_type'] = 'response'
            json_result['ok'] = True

            token_res = OrderedDict()
            token_res['_type'] = 'token'
            token_res['value'] = token

            # Get the dissection from dissector tool
            try:
                dissection = Dissector(pcap_path).dissect(eval(prot['name']))
            except TypeError as e:
                self.api_error('Dissector error: ' + str(e))
                return
            except pure_pcapy.PcapError:
                self.api_error(
                    "Expected 'pcap_file' to be a non empty pcap file"
                )
                return
            except:
                self.api_error(
                    "Couldn't read the temporary file %s and protocol is %s"
                    %
                    (
                        pcap_path,
                        prot['name']
                    )
                )
                return

            # Save the json dissection result into a file
            json_save = os.path.join(
                TMPDIR,
                token + '.json'
            )
            try:
                with open(json_save, 'w') as f:
                    json.dump(dissection, f)
            except:
                self.api_error("Couldn't write the json file")
                return

            # Add the token to the results
            dissection.insert(0, token_res)

            # The json result to return
            json_result['content'] = dissection

            # Here we will analyze the pcap file and get the results as json
            print(json.dumps(json_result))
            return

        # ######################## End of API part ######################### #

        # DEPRECATED
        # elif (self.path == "/submit"):

        #     if os.fork():
        #         # close the socket right now(because the
        #         # requesthandler may do a shutdown(), which triggers a
        #         # SIGCHLD in the child process)
        #         self.connection.close()
        #         return

        #     parser = BytesFeedParser()
        #     ct = self.headers.get("Content-Type")
        #     if not ct.startswith("multipart/form-data;"):
        #         self.send_error(400)
        #         return

        #     parser.feed(bytes("Content-Type: %s\r\n\r\n" % ct, "ascii"))
        #     parser.feed(self.rfile.read(int(self.headers['Content-Length'])))
        #     msg = parser.close()

        #     # agree checkbox is selected
        #     for part in msg.get_payload():
        #         if isinstance(part, email.message.Message):
        #             disposition = part.get("content-disposition")
        #             if disposition and 'name="agree"' in disposition:
        #                 agree = True
        #                 break
        #     else:
        #         agree = False

        #     # urifilter checkbox is selected
        #     for part in msg.get_payload():
        #         if isinstance(part, email.message.Message):
        #             disposition = part.get("content-disposition")
        #             if disposition and 'name="urifilter"' in disposition:
        #                 urifilter = True
        #                 break
        #     else:
        #         urifilter = False

        #     # content of the regex box
        #     for part in msg.get_payload():
        #         if isinstance(part, email.message.Message):
        #             disposition = part.get("content-disposition")
        #             if disposition and 'name="regex"' in disposition:
        #                 regex = part.get_payload()
        #                 if not regex:
        #                     regex = None
        #                 break
        #     else:
        #         regex = None

        #     # profile radio buttons
        #     for part in msg.get_payload():
        #         if isinstance(part, email.message.Message):
        #             disposition = part.get("content-disposition")
        #             if disposition and 'name="profile"' in disposition:
        #                 profile = part.get_payload()
        #                 break
        #     else:
        #         profile = "client"

        #     # receive the pcap file
        #     for part in msg.get_payload():
        #         if isinstance(part, email.message.Message):
        #             disposition = part.get("content-disposition")
        #             if disposition and 'name="file"' in disposition:
        #                 mo = re.search('filename="([^"]*)"', disposition)

        #                 orig_filename = mo.group(1) if mo else None

        #                 timestamp = time.strftime("%y%m%d_%H%M%S")

        #                 pcap_file = os.path.join(
        #                         (DATADIR if agree else TMPDIR),
        #                         "%s_%04d.dump" % (timestamp, job_id)
        #                 )
        #                 self.log_message("uploading %s(urifilter=%r, regex=%r)", pcap_file, urifilter, regex)
        #                 with open(pcap_file, "wb") as fd:
        #                     # FIXME: using hidden API(._payload) because it seems that there is something broken with the encoding when getting the payload using .get_payload()
        #                     fd.write(part._payload.encode("ascii", errors="surrogateescape"))

        #                 break
        #     else:
        #         self.send_error(400)
        #         return

        #     self.send_response(200)
        #     self.send_header("Content-Type", "text/html;charset=utf-8")
        #     self.end_headers()

        #     out = UTF8Wrapper(self.wfile)

        #     self.wfile.flush()

        #     os.dup2(self.wfile.fileno(), sys.stdout.fileno())

        #     try:
        #         exceptions = []
        #         analysis.analyse_file_html(pcap_file, orig_filename, urifilter, exceptions, regex, profile)
        #         for tc in exceptions:
        #             self.log_message("exception in %s", type(tc).__name__, append=tc.exception)
        #     except pure_pcapy.PcapError:
        #         print("Bad file format!")

        #     shutdown()

        # If we didn't manage to bind the request
        else:
            self.send_error(404)
            return

job_id = 0


__shutdown = False


def shutdown():
    global __shutdown
    __shutdown = True

for d in TMPDIR, DATADIR, LOGDIR:
    try:
        os.makedirs(d)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise


def reopen_log_file(signum, frame):
    global log_file
    log_file = open(os.path.join(LOGDIR, "webserver.log"), "a")



#!/usr/bin/env python3
#
#  (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import re

from ttproto.core.analyzer import TestCase, is_protocol, Node, Conversation, Capture
from ttproto.core.dissector import Frame
from ttproto.core.templates import All, Not, Any, Length
from ttproto.core.typecheck import *
from ttproto.core.lib.all import *
from urllib import parse
from ttproto.core.exceptions import Error

class NoStimuliFoundForTestcase(Error):
    """
    Error raised when no stimuli was defined for the testcase
    """
    pass

class FilterError(Error):
    """
    The error thrown when an error occurs into Filter object during its run
    """
    pass

class CoapPrivacyTestCase(TestCase):
    """
    The test case extension for privacy analysis on CoAP transactions.
    """

    @classmethod
    @typecheck
    def get_protocol(cls) -> is_protocol:
        """
        Get the protocol corresponding to this test case. This has to be
        implemented into the protocol's common test case class.

        :return: The protocol on which this TC will occur
        :rtype: Value
        """
        return CoAP

    @classmethod
    @typecheck
    def get_nodes_identification_templates(cls) -> list_of(Node):
        """
        Get the nodes of this test case. This has to be be implemented into
        each test cases class.

        :return: The nodes of this TC
        :rtype: [Node]

        .. note:: For CoAP it is simpler so we can define this function in this
                  class but for other protocols it can happend that we have to
                  define this inside each TC
        """
        return [
            Node('client', UDP(dport=5683)),
            Node('server', UDP(sport=5683))
        ]

    @classmethod
    @typecheck
    def get_test_purpose(cls) -> str:
        """
        Get the purpose of this test case

        :return: The purpose of this test case
        :rtype: str
        """
        if cls.__doc__:
            save = False
            for line in cls.__doc__.splitlines():
                if line.startswith('Objective'):
                    ret = line.split('*')[1]
                    save = True
                elif line.startswith('Configuration'):
                    return ' '.join(ret.split())
                elif save:
                    ret += line
        return ''

    @classmethod
    @typecheck
    def preprocess(
            cls,
            capture: Capture,
            expected_frames_pattern:list_of(Value)
    ) -> (list_of(Conversation), list_of(Frame)):
        """
        Preprocess and filter the frames of the capture into test case related conversations.

        :param Capture: The capture which will be filtered/preprocessed
        :return:
        """

        # Get informations from the test case
        protocol = CoapPrivacyTestCase.get_protocol()
        nodes = CoapPrivacyTestCase.get_nodes_identification_templates()

        conversations = []
        ignored = []

        if not nodes or len(nodes) < 2:
            raise ValueError('Expected at leaset two nodes declaration from the test case')
        if not protocol:
            raise ValueError('Expected a protocol under test declaration from the test case')

        # Get protocol related frames
        frames, ignored = Frame.filter_frames(capture.frames, protocol)

        # For privacy we dont need a preprocess for splitting the frames into conversations, we just analize it all
        c = Conversation(nodes)
        for f in frames:
            c.append(f)
        conversations.append(c)

        return conversations, ignored



#!/usr/bin/env python3
from ..common import *


class TD_COAP_CORE_14(CoAPTestCase):
    """
---
TD_COAP_CORE_14:
    cfg: CoAP_CFG_BASIC
    obj: Perform GET transaction containing several URI-Query options(CON mode)
    pre: "Server offers a /query resource with resource content is not empty"
    ref: '[COAP] 5.4.5, 5.10.2, 6.5'
    seq:
    -   s: "Client is requested to send a confirmable GET request with three\
            Query parameters (e.g. ?first=1&second=2&third=3) to the server\
            \u2019s resource"
    -   c:
        - 'The request sent by the client contains:'
        -   - Type = 0 (CON)
            - Code = 1 (GET)
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Uri-Path option "query"
        - 'and two options of Uri-Query, with values such as:'
        -   - first=1
            - second=2
    - c:
        - 'Server sends response containing:'
        -   - Code = 2.05 (Content)
            - Message ID = CMID, Token = CTOK
            - Content-format option
            - Non-empty Payload
    -   v: Client displays the response

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]

        .. note::
            Check the number/value of the uri query options or not?
        """
        return [
            CoAP(
                type='con',
                code='get',
                opt=Opt(CoAPOptionUriQuery(),
                        CoAPOptionUriQuery(),
                        CoAPOptionUriPath("query"),
                        superset=True,
                        )
            )
        ]

    def run(self):
        if self.urifilter:
            uri_query_opt = self.uri("/query?first=1&second=2")
        else:
            uri_query_opt = Opt(CoAPOptionUriQuery(), superset=True)

        self.match("client", CoAP(code="get", type="con", opt=uri_query_opt))
        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        opts = list(filter((lambda o: isinstance(o, CoAPOptionUriQuery)), self.coap["opt"]))

        if len(opts) < 2:
            self.set_verdict("inconclusive", "expect multiple UriQuery options")

        self.next_skip_ack()

        if self.match('server', CoAP(code=2.05, mid=CMID, tok=CTOK, pl=Not(b''))):
            self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat())), 'fail')



#!/usr/bin/env python3

from ..common import *


class TD_COAP_CORE_31(CoAPTestCase):
    """
---
TD_COAP_CORE_31:
    cfg: CoAP_CFG_BASIC
    obj: Perform CoAP Ping (CON mode)
    pre: (Should work with any CoAP server)
    ref: '[COAP] 4.3'
    seq:
    -   s:
        - 'Client is requested to send a "Ping" request with:'
        -   - Type = 0 (CON)
            - Code = 0 (empty)
    -   c:
        - 'The request sent by the client is four bytes and contains:'
        -   - Type=0 and Code=0
            - "Client-generated Message ID (\u2794 CMID)"
            - Zero-length Token
            - No payload
    -   c:
        - 'Server sends four-byte RST response containing:'
        -   - Type=3 and Code=0
            - Message ID = CMID
            - Zero-length Token
            - No payload
    -   v: Client displays that the "Ping" was successful
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]

        .. note::
            Check the number/value of the uri query options or not?
        """
        return [
            CoAP(type='con', code=0)  # Step 1
        ]

    def run(self):
        self.match("client", CoAP(type="con", code=0, tok=b"", pl=b""))
        CMID = self.coap["mid"]

        self.next_skip_ack()

        if self.match("server", CoAP(type=3)):
            self.match("server", CoAP(code=0, tok=b"", pl=b"", ), "fail")



from ..common import *

class TD_COAP_OBS_08(CoAPTestCase):
    """
---
TD_COAP_OBS_08:
    obj: Server cleans the observers list when observed resource content-format changes
    cfg: CoAP_CFG_BASIC
    ref: '[OBSERVE] 4.2.3'
    pre:
        - Client supports Observe option
        - Server supports Observe option
        - 'Server offers an observable resource /obs which changes periodically
        (e.g. every 5s) which produces confirmable notifications'

    seq:
    -   s: 'Client is requested to send to the server a confirmable GET
        request with observe option for resource /obs'

    -   c:
        - 'The request sent by client contains:'
        -   - Type = 0(CON)
            - Code = 1(GET)
            - Token value  = a value generated by the client
            - Observe option = empty

    -   c:
        - 'Server sends the response containing:'
        -   - Type = 2(ACK)
            - Code = 2.05(Content)
            - Content-format of the resource /obs
            - Token value = same as one found in the step 2
            - Observe option with a sequence number

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0 (CON)
            - Code = 2.05 (Content)
            - Content-format = same as one found in the step 3
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values

    -   c: Client displays the received information

    -   c: Client sends an ACK

    -   s: "Update the /obs resource of the server\u2019s resource
        with a new payload having a different Content-Format (either locally
        or by having another CoAP client perform a \\"DELETE\\" request)"
    -   c:
        - 'Server sends notification containing:'
        -   - Type = 0 (CON)
            - Code = 4.06 (Not Acceptable)
            - Token value = same as one found in the step 2
            - No Observe option any more

    -   v: Server does not send further notifications

    -   v: Client does not display further received information

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionObserve(0),CoAPOptionUriPath("obs"))),
            CoAP(type='con', code='post', opt=Opt(CoAPOptionUriPath("obs"), CoAPOptionContentFormat()))
        ]

    def run(self):
        request = CoAP(type="con", code="get", opt=self.uri("/obs", CoAPOptionObserve(0)))
        response = CoAP(type="ack", code=2.05, opt=Opt(CoAPOptionObserve(), CoAPOptionContentFormat()))

        # Step 2
        self.match("client", request)

        token = self.coap["tok"]
        uri = self.coap.get_uri()

        self.next()

        # Step 3
        if not self.match("server", response):
            raise self.Stop()
        self.match("server", CoAP(tok = token), "fail")

        content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
        index          = self.coap["opt"][CoAPOptionObserve]["val"]

        self.next()

        # we need to observe at least one notification
        # (to ensure that the server is observing the resource for the client)

        verdict_if_none = "inconclusive"

        # Optional as the 2nd stimulis may be done internally.
        stimulis = False

        # Step 4
        while not stimulis and self.match("server", CoAP(type="con", code=2.05, opt=Opt(CoAPOptionObserve())),verdict_if_none):
            self.match("server", CoAP(tok = token), "fail")
            self.match("server", CoAP(opt = Opt(CoAPOptionContentFormat(content_format))), "fail")
            new_index = self.coap["opt"][CoAPOptionObserve]["val"]

            self.set_verdict(("pass" if new_index > index else "fail"),
                "value of observe option must be increasing")

            new_content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
            if new_content_format != content_format:
                self.set_verdict(("fail"),
                    "The Content-Format changed but the server did not send\
                    an error as expected")

            self.next()

            stimulis = self.__check_stimulis(content_format)
            if not stimulis:
                # Step 6
                self.match("client", CoAP(type="ack", code=0),"fail")
                # now we have successfully observed a observe response
                self.next()
                verdict_if_none = None

            stimulis = self.__check_stimulis(content_format)

        if stimulis:
            verdict_if_none = "fail"
        else:
            verdict_if_none = None

        if self.match("server",CoAP (type="con",
                                     code=Any(134,160),
                                     opt = NoOpt(CoAPOptionObserve())
                                     ), verdict_if_none):
            self.match("server", CoAP(tok=token), "fail")
            self.next()

        # Step 9
        self.match("client", CoAP (type="ack", code=0), None)

    def __check_stimulis(self, content_format:str)->bool:
        # Step 7: Stimulus. This step is optional as changing the Content-Format
        # may be done internally by the server.
        # return a boolean that determine if we encountered this optional stimulis

        stimulis = False
        # Optional as stimulis may be done internally.
        CMID_stimulis = None
        CTOK_stimulis = None
        if self.match("client", CoAP(type='con', code='post',
        opt=Opt(CoAPOptionUriPath("obs"), CoAPOptionContentFormat())), None):
            new_content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
            CMID_stimulis = self.coap["mid"]
            CTOK_stimulis = self.coap["tok"]
            self.set_verdict(("pass" if new_content_format != content_format
                else None),"Stimulus: POST performed with new Content-Format.")
            # stimulis = True
            self.next()

        # We assume that the confirmation come immediatly in the current frame as
        # the preprocess did some reoordering.
        if self.match("server", CoAP(type="ack", code=2.04, tok=CTOK_stimulis,
        mid=CMID_stimulis), None):
            self.set_verdict("pass", "Stimulus: Confirmation from server that \
POST suceeded with new Content-Format")
            self.next()
            stimulis = True

        return stimulis



from ..common import *


class TD_IPSO_3304_02(CoAPTestCase):
    """
testcase_id: TD_IPSO_3304_02 
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m 
configuration: LWM2M_CFG_01 
objective: Creating a new Instance of the Humidity Sensor Object in JSON format
pre_conditions:
  - Device is registred with the Server
  - The current values of the Server Object (ID:1) Instance 0, are saved on the Server
  - The Client supports the Configuration C.3 (A superset of C.1 where server objects implements {Default Minimum PEriod, Default Maximum Period, Disable Timeout} additional resources)  
  - - LWM2M Server Object (ID = 1) Instance 0 with mandatory resources and Short Server ID = 1
  - - LWM2M Security Object (ID = 0) Instance 0 with mandatory resources and Bootstrap Server = False
  - - LWM2M Device Object (ID = 3) Instance 0 with mandatory resources. 
  - The Client supports IPSO Humidity Sensor (ID:3304) with mandatory resources. 
sequence:
  - step_id: 'TD_IPSO_3304_02_step_01'
    type: stimuli
    node: lwm2m_server
    description:
      - 'LwM2M server sends a CREATE request (CoAP POST) on Humidity Sensor object'
      - - Type = 0 (CON)
        - Code = 2 (POST)

  - step_id: 'TD_IPSO_3304_02_step_02'
    type: check
    description:
      - 'Sent POST request contains'
      - - Type=0 and Code=2
        - Non-empty payload = The representation of new Humidity object instance
        - content-format=application/vnd.oma.lwm2m+json
        - URI-Path option= /3304

  - step_id: 'TD_IPSO_3304_02_step_03'
    type: check
    description:
      - 'LwM2M client sends response containing'
      - - Code = 2.04 (Changed)
        - Location-Path option= the location of the created instance (LP)

  - step_id: 'TD_IPSO_3304_02_step_04'
    type: verify
    node: lwm2m_server
    description:
      - 'LwM2M server indicates successful operation'

  - step_id: 'TD_IPSO_3304_02_step_05'
    type: stimuli
    node: lwm2m_server
    description:
      - 'LwM2M server sends a READ request (CoAP GET) on new created Humidity Sensor object'
      - - Type = 0 (CON)
        - Code = 1 (GET)

  - step_id: 'TD_IPSO_3304_02_step_06'
    type: check
    description:
      - 'Sent GET request contains'
      - - Type=0 and Code=1
        - Accept option=application/vnd.oma.lwm2m+json
        - URI-Path option= LP

  - step_id: 'TD_IPSO_3304_02_step_07'
    type: check
    description:
      - 'LwM2M client sends response containing'
      - - Code = 2.05 (Content)
        - content-format=application/vnd.oma.lwm2m+json
        - Non-empty payload = A serialized representation of new instance 

  - step_id: 'TD_IPSO_3304_02_step_08'
    type: verify
    node: lwm2m_server
    description:
      - 'LwM2M server indicates successful operation'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='post'), CoAP(type='con', code='get')]

    def run(self):
        self.match('server', CoAP(type='con', code='post', pl=Not(b''), opt=self.uri('/3304')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat('11543'))), 'fail')
        

        self.next()

        self.match('client', CoAP(code=Any(65, 68), pl=(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionLocationPath())), 'fail')

        OPTS = self.coap['opt']
        LP = OPTS[CoAPOptionLocationPath]
        LPVAL = RI[2]

        self.next()
        
    
        self.match('server', CoAP(type='con', code='get', pl=(b''), opt=self.uri(LPVAL)), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('11543'))), 'fail')
        
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('11543'))), 'fail')
        
        validation = validate(str(self.coap['pl']),'3304')
        self.set_verdict(validation, 'payload validation')



from ..common import *


class TD_LWM2M_1_INT_201(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_201 
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m 
configuration: LWM2M_CFG_01 
objective:   
  - Querying the following data on the client (Device Object = ID 3) in text plain format 
  - - Manufacturer 
    - Model number 
    - Serial number 
pre_conditions: 
  - Device is registered at the LWM2M server
  - Client is supporting Plain Text data format (0)
  - The Client supports the minimum Configuration C.1.
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
sequence: 
  - step_id: 'TD_LWM2M_1.0_INT_201_step_01'
    type: stimuli 
    node : lwm2m_server 
    description: 
      - 'Server sends a READ request (COAP GET) on device object’s Manufacturer resource'       
      - - Type = 0 (CON)         
        - Code = 1 (GET) 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_02' 
    type: check 
    description:
      - 'The request sent by the server contains'   
      - - Type=0 and Code=1  
          - Accept option = text/plain
          - Uri-Path option = 3/0/0

  - step_id: 'TD_LWM2M_1.0_INT_201_step_03'
    type: check
    description:       
      - 'Client sends response containing'         
      - - Code = 2.05 (Content)           
        - Non-empty Payload 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_04' 
    type: verify 
    node: lwm2m_server 
    description:         
      - 'Requested data is successfully displayed' 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_05' 
    type: stimuli 
    node : lwm2m_server 
    description: 
      - 'Server sends a READ request (COAP GET) on device object’s Model Number resource'       
      - - Type = 0 (CON)         
        - Code = 1 (GET) 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_06' 
    type: check 
    description:       
      - 'The request sent by the server contains'       
      - - Type=0 and Code=1         
        - Accept option = text/plain                 
        - Uri-Path option = 3/0/1 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_07' 
    type: check 
    description:         
      - 'Client sends response containing'         
      - - Code = 2.05 (Content)
        - Non-empty Payload
 
  - step_id: 'TD_LWM2M_1.0_INT_201_step_08' 
    type: verify 
    node: lwm2m_server 
    description:         
      - 'Requested data is successfully displayed' 

  - step_id: 'TD_LWM2M_1.0_INT_201_step_09' 
    type: stimuli 
    node : lwm2m_server 
    description: 
      - 'Server sends a READ request (COAP GET) on device object’s Serial Number resource'
      - - Type = 0 (CON)         
        - Code = 1 (GET)
 
  - step_id: 'TD_LWM2M_1.0_INT_201_step_10' 
    type: check 
    description:       
      - 'The request sent by the server contains'       
      - - Type=0 and Code=1         
        - Accept option = text/plain                 
        - Uri-Path option = 3/0/2
 
  - step_id: 'TD_LWM2M_1.0_INT_201_step_11' 
    type: check 
    description:         
      - 'Client sends response containing'         
      - - Code = 2.05 (Content)           
        - Non-empty Payload

  - step_id: 'TD_LWM2M_1.0_INT_201_step_12' 
    type: verify 
    node: lwm2m_server 
    description:         
      - 'Requested data is successfully displayed'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='get')]

    def run(self):
        self.match('server', CoAP(type='con', code='get', opt=self.uri('/3/0/0')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('40'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')

        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/3/0/1')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('40'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')
        
        self.next()

        self.match('server', CoAP(type='con', code='get', opt=self.uri('/3/0/2')), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionAccept('40'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=2.05, pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionContentFormat('40'))), 'fail')



from ttproto.core.analyzer import Analyzer
from ttproto.core.templates import Length, Regex, All, Range, Any, AnyValue
import unittest
import os
import logging

logging.basicConfig()
logger = logging.getLogger(__name__)
# Change logging level here.
logger.setLevel(os.environ.get('LOG_LEVEL', logging.INFO))
default_expected_verdict_if_none_specified = "pass"


class TestAnalysisInteropTestCase(unittest.TestCase):

    def setUp(self):
        self.pcap_dir_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../test_dumps')
        self.pcap_path_list = os.listdir(self.pcap_dir_path)
        logger.info(self.pcap_path_list)

    def get_pcaps(self):
        for entry in self.pcap_path_list:
            full_path = os.path.join(self.pcap_dir_path, entry)
            if os.path.isfile(full_path) and full_path.endswith('.pcap'):
                yield full_path

    def __get_test_case_from_full_path(self, pcap_path):
        if 'PASS' in pcap_path:
            return pcap_path[pcap_path.find('TD'):(pcap_path.find('PASS') - 1)]
        elif 'FAIL' in pcap_path:
            return pcap_path[pcap_path.find('TD'):(pcap_path.find('FAIL') - 1)]
        elif 'INCONCLUSIVE' in pcap_path:
            return pcap_path[pcap_path.find('TD'):(pcap_path.find('INCONCLUSIVE') - 1)]
        else:
            return None

    def test_td_coap(self):
        for pcap_name in self.get_pcaps():
            expected_verdict = None
            if 'PASS' in pcap_name:
                expected_verdict = 'pass'
            elif 'FAIL' in pcap_name:
                expected_verdict = 'fail'
            elif 'INCONCLUSIVE' in pcap_name:
                expected_verdict = 'inconclusive'
            else:
                if default_expected_verdict_if_none_specified == "error":
                    raise Exception("No verdict in name with pcap filename " + pcap_name)
                else:
                    expected_verdict = default_expected_verdict_if_none_specified
            analyzer = Analyzer('tat_coap')

            test_case = self.__get_test_case_from_full_path(pcap_name)
            if test_case is None:
                logging.info("Ignored file " + pcap_name + " because we couldn't determine the relevant test case.")
                continue

            logging.info("Testing " + test_case + "\n With file " + pcap_name)

            tc_name, verdict, rev_frames, log, partial_verdicts, exception_info = analyzer.analyse(pcap_name, test_case)
            assert verdict == expected_verdict, log

            logging.info(test_case + " PASSED.")



from ..common import *


class TD_COAP_CORE_13 (CoAPTestCase):
    """
---
TD_COAP_CORE_13:
    cfg: CoAP_CFG_BASIC
    obj: Perform GET transaction containing several URI-Path options (CON
        mode)
    pre: Server offers a /seg1/seg2/seg3 resource with resource content
        is not empty
    ref: '[COAP] 5.4.5, 5.10.2, 6.5'
    seq:
    -   s: "Client is requested to send a confirmable GET request to server\
            \u2019s resource"
    -   c:
        - 'The request sent by the client contains:'
        -   - Type = 0 (CON)
            - Code = 1 (GET)
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Uri-Path option "test"
        - 'and three options of type Uri-Path, with the values:'
        -   - seg1
            - seg2
            - seg3
    -   c:
        - 'Server sends response containing:'
        -   - Code = 2.05 (Content)
            - Message ID = CMID, Token = CTOK
            - Content-format option
            - Non-empty Payload
    -   v: Client displays the response
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get',
                 opt=Opt(
                        CoAPOptionUriPath("seg1"),
                        CoAPOptionUriPath("seg2"),
                        CoAPOptionUriPath("seg3")
                         )
                 )
        ]

    def run(self):
        if self.urifilter:
            uri_path_opt = self.uri("/seg1/seg2/seg3")
        else:
            uri_path_opt = Opt(CoAPOptionUriPath(), superset=True)

        self.match("client", CoAP(code="get",type="con",opt=uri_path_opt))

        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        opts = list (filter ((lambda o: isinstance (o, CoAPOptionUriPath)), self.coap["opt"]))

        if len (opts) > 1:
            self.set_verdict ("pass", "multiple UriPath options")
        else:
            self.set_verdict ("inconclusive", "only one UriPath option")

        # TODO: move this checks outside the testcases
        for o in opts:
            if "/" in str (o["val"]):
                self.set_verdict ("fail", "option %s contains a '/'" % repr (o))

        self.next_skip_ack()

        if self.match(
            'server',
            CoAP(code=2.05, type = Any (CoAPType("con"), "ack"), mid=CMID, tok=CTOK, pl=Not(b''))
        ):
            self.match(
                'server',
                CoAP(opt=Opt(CoAPOptionContentFormat())),
                'fail'
            )



#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from	ttproto.core.typecheck	import *
from	ttproto.core.data	import *
from	ttproto.core		import exceptions

class UnionClass (type):
	@typecheck
	def __new__ (cls, name, bases, dct, types: optional(iterable) = None):

		if types is None:
			if len(bases) == 0:
				raise exceptions.Error ("UnionClass requires the 'types' parameter")

			# happens when the union is derived
			# -> just build the class
			for b in bases:
				if isinstance (b, UnionClass):
					break
			else:
				raise exceptions.Error ("UnionClass requires the 'types' parameter or that the type is derived from a type built with UnionClass")
			return type.__new__ (cls, name, bases, dct)

		assert len (bases) == 0 # TODO: maybe relax this

		bases = Value,
		result = type.__new__ (cls, name, bases, dct)
		result.__types = tuple (get_type(t) for t in types)
		return result

	def __init__ (self, name, bases, dct, types = None):
		type.__init__ (self, name, bases, dct)

	def __instancecheck__ (self, instance):
		return isinstance (instance, self.__types)

	def __subclasscheck__ (self, subclass):
		return issubclass (subclass, self.__types) or self in subclass.__mro__




#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from .common import *
from .testsrouter import *
from . import implemR


################################################################################
# Test_LPND_1.4.1a
################################################################################
@testcase(SixLoWPAN_ND)
def Test_LPND_1_4_1a(NUT , TIMER1 , HOST1 , LBR1 , PRFX1 , link1, action):

	t = Timer()

	unexpected_packets.activate( link1 )
	timeout.activate( t )

	rs = pack (
		IPv6( src = HOST1.lladdr , dst = NUT.lladdr ) /
		ICMPv6RSol( opt = [
			ICMPv6SLL( hw=HOST1.hwaddr )
			]
		)
	)

	step (1, "We perform the common test setup 1.2" )
	CommonTestSetup_1_2(LBR1 , NUT , PRFX1 , TIMER1, link1)

	step (2, "We send a router solicitation ")
	link1.send(rs)

	step (3, "We expect a router advertisement with a valid Authoritative Border Router Option" )
	t.start(TIMER1)

	link1.receive (
		IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
		ICMPv6Adv( opt = superop ( [
			SixABRO ( lbr = LBR1.addr )
			] )
		)
	)
	set_verdict("pass")


################################################################################
# Test_LPND_1.4.1b
################################################################################
@testcase(SixLoWPAN_ND)
def Test_LPND_1_4_1b(NUT , TIMER1 , HOST1 , LBR1 , LBR2 , PRFX1 , PRFX2 , link1, action):

	t = Timer()

	unexpected_packets.activate( link1 )
	timeout.activate( t )

	rs = pack (
		IPv6( src = HOST1.lladdr , dst = NUT.lladdr ) /
		ICMPv6RSol( opt = [
			ICMPv6SLL( hw=HOST1.hwaddr )
			]
		)
	)

	ra = pack (
		IPv6( src = LBR2.lladdr , dst = NUT.lladdr ) ,
		ICMPv6RAdv( opt = [
				SixABRO(lbr=LBR2.addr) ,
				SixPI( pf=PRFX2 , vlt = 120 )
			]
		)
	)

	step (1, "We perform the common test setup 1.2" )
	CommonTestSetup_1_2(LBR1 , NUT , PRFX1 , TIMER1, link1)

	step (2, "We send a router advertisement with an Authoritative Border Router Option")
	link1.send(ra)

	step (3, "We send a router solicitation")
	link1.send(rs)

	step (4, "We expect 2 two router advertisement with a valid Authoritative Border Router Option" )
	t.start(TIMER1)

	receive_unordered ( link1 , 
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 )
				] )
			)
		),
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR2.addr) ,
					SixPI( pf=PRFX2 , vlt = 120 )
				] )
			)
		)
	)
#	set_verdict("pass") handled by receive_unordered()

################################################################################
# Test_LPND_1.4.1c
################################################################################
@testcase(SixLoWPAN_ND)
def Test_LPND_1_4_1c(NUT , TIMER1 , HOST1 , LBR1 , LBR2 , PRFX1 , link1, action):

	t = Timer()

	unexpected_packets.activate( link1 )
	timeout.activate( t )

	rs = pack (
		IPv6( src = HOST1.lladdr , dst = NUT.lladdr ) /
		ICMPv6RSol( opt = [
			ICMPv6SLL( hw=HOST1.hwaddr )
			]
		)
	)

	ra = pack (
		IPv6( src = LBR2.lladdr , dst = NUT.lladdr ) ,
		ICMPv6RAdv( opt = [
				SixABRO(lbr=LBR1.addr) ,
				SixPI( pf=PRFX1 , vlt = 120 )
			]
		)
	)

	step (1, "We perform the common test setup 1.2" )
	CommonTestSetup_1_2(LBR1 , NUT , PRFX1 , TIMER1, link1)

	step (2, "We send a router advertisement with an Authoritative Border Router Option")
	link1.send(ra)

	step (3, "We send a router solicitation")
	link1.send(rs)

	step (4, "We expect two router advertisement with two identical Authoritative Border Router Option" )
	t.start(TIMER1)

	receive_unordered ( link1 , 
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 )
				] )
			)
		),
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 )
				] )
			)
		)
	)
#	set_verdict("pass") handled by receive_unordered()

################################################################################
# Test_LPND_1.4.1d
################################################################################
@testcase(SixLoWPAN_ND)
def Test_LPND_1_4_1d(NUT , TIMER1 , HOST1 , LBR1 , LBR2, PRFX1 , PRFX2 , link1, action):

	t = Timer()

	unexpected_packets.activate( link1 )
	timeout.activate( t )

	rs = pack (
		IPv6( src = HOST1.lladdr , dst = NUT.lladdr ) /
		ICMPv6RSol( opt = [
			ICMPv6SLL( hw=HOST1.hwaddr )
			]
		)
	)

	ra = pack (
		IPv6( src = LBR2.lladdr , dst = NUT.lladdr ) ,
		ICMPv6RAdv( opt = [
				SixABRO(lbr=LBR2.addr) ,
				SixPI( pf=PRFX2 , vlt = 120 ) ,
				SixCO( cid = 2 , c = 0 , clen = 64, pf = PRFX2)
			]
		)
	)

	step (1, "We perform the common test setup 1.2b" )
	CommonTestSetup_1_2b(LBR1 , NUT , PRFX1 , TIMER1, link1)

	step (2, "We send a router advertisement with an Authoritative Border Router Option")
	link1.send(ra)

	step (3, "We send a router solicitation")
	link1.send(rs)

	step (4, "We expect 2 two router advertisement with a valid Authoritative Border Router Option" )
	t.start(TIMER1)

	receive_unordered ( link1 , 
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 ) ,
					SixCO( cid = 1 , c = 1 , clen = 64, pf = PRFX1)
				] )
			)
		),
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR2.addr) ,
					SixPI( pf=PRFX2 , vlt = 120 ) ,
					SixCO( cid = 2 , c = 0 , clen = 64, pf = PRFX2)
				] )
			)
		)
	)
#	set_verdict("pass") handled by receive_unordered()

################################################################################
# Test_LPND_1.4.1e
################################################################################
@testcase(SixLoWPAN_ND)
def Test_LPND_1_4_1e(NUT , TIMER1 , HOST1 , LBR1 , LBR2 , PRFX1 , link1, action):

	t = Timer()

	unexpected_packets.activate( link1 )
	timeout.activate( t )

	rs = pack (
		IPv6( src = HOST1.lladdr , dst = NUT.lladdr ) /
		ICMPv6RSol( opt = [
			ICMPv6SLL( hw=HOST1.hwaddr )
			]
		)
	)

	ra = pack (
		IPv6( src = LBR2.lladdr , dst = NUT.lladdr ) ,
		ICMPv6RAdv( opt = [
				SixABRO(lbr=LBR1.addr) ,
				SixPI( pf=PRFX1 , vlt = 60 ) ,
				SixCO( cid = 1 ,clen = 64, pf = PRFX1)
			]
		)
	)

	step (1, "We perform the common test setup 1.2b" )
	CommonTestSetup_1_2b(LBR1 , NUT , PRFX1 , TIMER1, link1)

	step (2, "We send a router advertisement with an Authoritative Border Router Option")
	link1.send(ra)

	step (3, "We send a router solicitation")
	link1.send(rs)

	step (4, "We expect a router advertisement with a valid Authoritative Border Router Option" )
	t.start(TIMER1)

	receive_unordered ( link1 , 
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 ) ,
					SixCO( cid = 1 ,clen = 64, pf = PRFX1)
				] )
			)
		),
		(
			IPv6( src = NUT.lladdr , dst = HOST1.lladdr ) /
			ICMPv6RAdv( opt = superop ( [
					SixABRO(lbr=LBR1.addr) ,
					SixPI( pf=PRFX1 , vlt = 60 ) ,
					SixCO( cid = 1 ,clen = 64, pf = PRFX1)
				] )
			)
		)
	)
	set_verdict("pass")


################################################################################
# Main()
################################################################################

if __name__ == "__main__":
	with LoggerGroup ([ConsoleLogger(), HTMLLogger()]) as l:
		Logger.set_default (l)
#		ts = TestSession ([ init, Test_LPND_1_3_3c])
#		ts.set_config(SixLoWPAN_ND, SixLoWPAN_ND("router"))
#		ts.run()
		run_all_testcases({SixLoWPAN_ND: SixLoWPAN_ND("router")})



from ..common import *


class TD_COAP_OBS_05(CoAPTestCase):
    """
---
TD_COAP_OBS_05:
    obj: Server detection of deregistration (client OFF)
    cfg: CoAP_CFG_BASIC
    ref: '[OBSERVE] 4.5 §2'
    pre:
        - Client supports Observe option
        - Server supports Observe option
        - "Server offers an observable resource /obs which changes periodically (e.g. every
        5s) which produces confirmable notifications"

    seq:
    -   s:
        - 'Client is requested to send to the server a confirmable GET
        request with observe option for resource /obs'

    -   c:
        - 'The request sent by client contains:'
        -   - Type = 0 (CON)
            - Code = 1 (GET)
            - Token value  = a value generated by the client
            - Observe option = empty

    -   c:
        - 'Server sends the response containing:'
        -   - Type = 2 (ACK)
            - Code = 2.05 (Content)
            - Content-format of the resource /obs
            - Token value = same as one found in the step 2
            - Observe option with a sequence number

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0 (CON)
            - Code = 2.05 (Content)
            - Content-format = same as one found in the step 3
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values

    -   c: Client displays the received information

    -   c: Client sends an ACK

    -   s: Client is switched off

    -   c:
        - Server’s confirmable responses are not acknowledged
        - Server’s retransmissions have an updated Observe option value

    -   c: "Server can keep retransmitting the responses until at least
        Max-Age seconds after the first un-acknowledged response."

    """
    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionObserve(0),CoAPOptionUriPath("obs")))
        ]

    def run (self):
        request = CoAP (type="con", code="get",	opt=self.uri("/obs", CoAPOptionObserve (0)))
        response= CoAP (type="ack", code=2.05, opt=Opt(CoAPOptionObserve(), CoAPOptionContentFormat()))
        notif   = CoAP (type="con", code=2.05, opt=Opt(CoAPOptionObserve(), CoAPOptionContentFormat()))

        # Step 2
        self.match("client", request)

        token = self.coap["tok"]
        uri = self.coap.get_uri()

        self.next()

        # Step 3
        if not self.match("server", response):
            raise self.Stop()
        self.match("server", CoAP(tok = token), "fail")

        content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
        index          = self.coap["opt"][CoAPOptionObserve]["val"]

        last_maxage = self.coap["opt"][CoAPOptionMaxAge]["val"]

        self.next()
        last_ts = self._frame.timestamp
        # Step 4
        unacknowledged_notification = False
        while self.match("server", notif, None) and not unacknowledged_notification:
            self.match("server", CoAP(tok = token), "fail")
            self.match("server", CoAP(opt = Opt(CoAPOptionContentFormat(content_format))), "fail")

            # increasing observe value
            new_index = self.coap["opt"][CoAPOptionObserve]["val"]
            self.set_verdict(("pass" if new_index > index else "fail"),
                "value of observe option must be increasing")
            index = new_index

            maxage = self.coap["opt"][CoAPOptionMaxAge]["val"]

            self.next()

            if not self.match("client", CoAP(type = "ack"), None):
                unacknowledged_notification = True

            last_maxage = maxage
            last_ts = self._frame.timestamp
            self.next()

        nb_nonack = 1
        while self.match("server", notif, None):
            self.match("server", CoAP(tok = token), "fail")
            self.match("server", CoAP(opt = Opt(CoAPOptionContentFormat(content_format))), "fail")

            nb_nonack += 1
            last_retrans_ts = self._frame.timestamp
            # self.next()
            if not self.next(optional = True):
                break

        if nb_nonack == 1:
            self.set_verdict("inconclusive", "server must retransmit non-acknowledged notifications")
        else:
            self.set_verdict("pass" if (last_retrans_ts - last_ts) >= last_maxage else "inconclusive", "server should keep retransmitting at least until Max-Age (%d) seconds" % last_maxage)



from ..common import *


class TD_M2M_NH_20(CoAPTestCase):
    """
---
TD_M2M_NH_20:
    cfg: M2M_CFG_01
    obj: AE discovers accessible resources residing in Registrar CSE limiting the number of matching resources to the specified value.
    pre: CSEBase resource has been created in CSE with name {CSEBaseName}.
    ref: 'TS-0001, clause 10.2.6 ; TS-0004, clause 7.2.3.13'
    seq:
    -   s:
        - 'AE is requested to send a Discovery request in order to discover at most 2 resources in registrar CSE with:'
        -   - Type = 0(CON)
            - Code = 1(GET)
    -   c:
        - 'The request sent by AE contains:'
        -   - Type=0 and Code=1
            - oneM2M-FR option=AE-ID
            - oneM2M-RQI option=token-string (CRQI)
            - UriQuery option="fu=1"
            - UriQuery option="lim=2"
            - Empty payload
    -   c:
        - 'Registrar CSE sends response containing:'
        -   - Code = 2.05(Content)
            - oneM2M-RSC option=2000
            - oneM2M-RQI option=CRQI
            - Content-format option
            - Non-empty Payload
    -   v: AE indicates successfull operation
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='get')]

    def run(self):
        
        self.match('client', CoAP(type='con', code='get', pl=Not(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionOneM2MFrom())), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionUriQuery('fu=1'))), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionUriQuery('lim=2'))), 'fail')
        if self.match('client', CoAP(opt=Opt(CoAPOptionOneM2MRequestIdentifier())), 'fail'): 

            CMID = self.coap['mid']
            CTOK = self.coap['tok']
            OPTS = self.coap['opt']
            RI = OPTS[CoAPOptionOneM2MRequestIdentifier]
            RIVAL = RI[2]

            self.next()

            self.match('server', CoAP(code=2.05, mid=CMID, tok=CTOK, pl=Not(b'')), 'fail')
            self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat())), 'fail')
            self.match('server', CoAP(opt=Opt(CoAPOptionOneM2MResponseStatusCode('2000'))), 'fail')
            self.match('server', CoAP(opt=Opt(CoAPOptionOneM2MRequestIdentifier(RIVAL))), 'fail')



#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from . import local_config
from . import implem
from . import implembR
from . import implemR
from .common import *

from ttproto.core import clock

class SUTActionEmulated (SUTAction):
	def run (self):
		# TODO
		pass
	
	def interrupt (self):
		pass


class SixLoWPAN_ND (Config):

	def __init__ (self, profil):
		Config.__init__ (self)

		if not local_config.EMULATION_MODE:
			self.__tap_port = SocatMessagePort("UDP:localhost:13000", Ieee802154)
		else:
			
			clock.Clock.set_instance (clock.SimulatedClock())
			
			#If we want to lunch the host implementation (implem.py)
			if( profil == "host") :
				self.__implementation = implem.SixLowpanImplementation (local_config.NUT_hwaddr)

			#If we want to lunch the router implementation (implemR.py)
			elif (profil == "router"):
				self.__implementation = implemR.SixLowpanImplementation (local_config.NUT_hwaddr)

			#If we want to lunch the border router implementation (implembR.py)
			elif (profil == "borderRouter") :
				self.__implementation = implembR.SixLowpanImplementation (local_config.NUT_hwaddr)


			self.__implementation.daemon = True
			self.__tap_port = self.__implementation.get_message_port()

			self.__implementation.start()

		class NutClass:
			pass
		NUT = NutClass()

		class TestNodeClass:
			pass
		LBR1 = TestNodeClass()
		LBR2 = TestNodeClass()
		LR1 = TestNodeClass()
		LR2 = TestNodeClass()
		HOST1 = TestNodeClass()
		HOST2 = TestNodeClass()

		NUT.addr	= IPv6Address(local_config.NUT_addr)
		NUT.lladdr	= IPv6Address(local_config.NUT_lladdr)
		NUT.hwaddr	= local_config.NUT_hwaddr
		NUT.eui64	= local_config.NUT_eui64

		LBR1.addr	= IPv6Address(local_config.LBR1_addr)
		LBR1.lladdr	= IPv6Address(local_config.LBR1_lladdr)
		LBR1.hwaddr	= local_config.LBR1_hwaddr
		LBR1.eui64	= local_config.LBR1_eui64

		LBR2.addr	= IPv6Address(local_config.LBR2_addr)
		LBR2.lladdr	= IPv6Address(local_config.LBR2_lladdr)
		LBR2.hwaddr	= local_config.LBR2_hwaddr
		LBR2.eui64	= local_config.LBR2_eui64

		LR1.addr	= IPv6Address(local_config.LR1_addr)
		LR1.lladdr	= IPv6Address(local_config.LR1_lladdr)
		LR1.hwaddr	= local_config.LR1_hwaddr
		LR1.eui64	= local_config.LR1_eui64

		LR2.addr	= IPv6Address(local_config.LR2_addr)
		LR2.lladdr	= IPv6Address(local_config.LR2_lladdr)
		LR2.hwaddr	= local_config.LR2_hwaddr
		LR2.eui64	= local_config.LR2_eui64

		HOST1.addr	= IPv6Address(local_config.HOST1_addr)
		HOST1.lladdr	= IPv6Address(local_config.HOST1_lladdr)
		HOST1.hwaddr	= local_config.HOST1_hwaddr
		HOST1.eui64	= local_config.HOST1_eui64

		HOST2.addr	= IPv6Address(local_config.HOST2_addr)
		HOST2.lladdr	= IPv6Address(local_config.HOST2_lladdr)
		HOST2.hwaddr	= local_config.HOST2_hwaddr
		HOST2.eui64	= local_config.HOST2_eui64

		TIMER1 = local_config.TIMER1
		TIMER2 = local_config.TIMER2
		SLEEP1 = local_config.SLEEP1

		PRFX1 = local_config.PRFX1
		PRFX2 = local_config.PRFX2

		PANID = local_config.PANID

		self["NUT"] = NUT
		self["LBR1"] = LBR1
		self["LBR2"] = LBR2
		self["LR1"] = LR1
		self["LR2"] = LR2

		self["TIMER1"]=TIMER1
		self["TIMER2"]=TIMER2
		self["SLEEP1"]=SLEEP1

		self["PRFX1"]=PRFX1
		self["PRFX2"]=PRFX2

		self["PANID"]=PANID

		self["HOST1"] = HOST1
		self["HOST2"] = HOST2

		self["action"] = SUTActionEmulated if local_config.EMULATION_MODE else SUTActionManual

	def __enter__ (self):
		link1 = SixLowpanMessagePort (self["PANID"], self.__tap_port)
		self["link1"] = link1

		if local_config.EMULATION_MODE:
			clock.Clock.get_instance().reset()

			self.__implementation.reset()

			clock.Clock.get_instance().start()

		
	def __exit__ (self, a, b, c):
		self["link1"].disconnect()

		if local_config.EMULATION_MODE:
			clock.Clock.get_instance().reset()



#!/usr/bin/env python3

from ttproto.ts_coap.proto_specific import CoAPTestcase
from ttproto.ts_coap.proto_templates import *


class TD_COAP_CORE_29(CoAPTestcase):
    """Identifier:
TD_COAP_CORE_29
Objective:
Perform GET transaction with responses containing the Max-Age option (Reverse
proxy)
Configuration:
CoAP_CFG_03
References:
[1] clause 5.8.1,5.10.6,5.9.1.3,5.9.1.5, 8.2.2,8.2.1,10.2.2,11.2

Pre-test
conditions:
•	Proxy offers a cache
•	Proxy is configured as a reverse-proxy for the server
•	Servers resource vary in time and supports Max-Age option
•	Proxy’s cache is cleared
•	Server offers a resource /validate that varies in time, with a Max-Age set to 30s

Test Sequence:
Step
Type
Description

1
Stimulus
A confirmable GET request is sent to Proxy from Client


2
Check
Proxy Sends request containing:
•	Type = 0 (CON)
•	Code = 1 (GET)


3
Check
Server sends response containing:
•	Code = 69 (2.05 Content)
•	Option type = ETag
•	Option value = ETag value
•	Option type = Max-age
•	Option value
•	Not empty Payload


4
Verify
Proxy forwards response to client

5
Stimulus
A confirmable GET request is sent to proxy from Client before
Max-Age expires

6
Check
Proxy dos not forward any request to the server

7
Check
Proxy sends response to client

8
Verify
Response contains:
•	Option type = Max-age
•	Option Value = new Max-age
•	Payload cached
	"""
    reverse_proxy = True

    def run(self):
        # Step 2
        self.match_coap("client", CoAP(type="con", code="get",
                                       opt=All(
                                           Opt(CoAPOptionUriPath("validate")),
                                           NoOpt(CoAPOptionETag()),
                                       )))

        self.next_skip_ack()

        if not self.match_coap("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                              code=2.05,
                                              opt=Opt(CoAPOptionETag(), CoAPOptionMaxAge()),
                                              pl=Not(b""))):
            raise self.Stop()

        maxage = self.frame.coap["opt"][CoAPOptionMaxAge]["val"]

        ts = self.frame.ts

        self.next_skip_ack(optional=True)

        while self.chain(optional=True):
            interval = self.frame.ts - ts

            if interval >= maxage:
                break

            if self.match_coap("client", CoAP(type="con", code="get",
                                              opt=Opt(CoAPOptionUriPath("validate"))),
                               None):
                raise self.setverdict("inconclusive",
                                      "Proxy sent a new GET request after %.1f seconds whereas Max-Age is set to %d seconds" % (
                                      interval, maxage))
                raise self.Stop()

            while self.next(optional=True):
                pass

        self.setverdict("pass", "No further GET requests were observed within Max-Age (%d) seconds" % maxage)



import os
import json
import logging
import unittest

from tests.test_tools.struct_validator import StructureValidator

from ttproto import TMPDIR
from ttproto.core.packet import PacketValue
from ttproto.core.lib.inet.coap import CoAP
from ttproto.core.dissector import Capture, get_dissectable_protocols
from ttproto.utils.pcap_filter import openwsn_profile_filter


class DissectPcapTestCase(unittest.TestCase):
    """
    Test pcap dissections using as input pcap files.

    This class can be inherited, and child class may have
    PCAP_FILES_DISSECTION_DIRS and PROTO_CLASS_FILTER
    overriden for running test over particular protocols.
    """

    # #################### Tests parameters #########################

    # File path
    PCAP_FILES_DISSECTION_DIRS = [
        'tests/test_dumps/lwm2m_pro',
    ]

    PROTO_CLASS_FILTER = CoAP  # we can override with None :)

    # Create a struct checker object
    struct_validator = StructureValidator()

    # #################### Init and deinit functions #########################
    def setUp(self):
        """
            Initialize the dissector instance
        """

        self.pcap_for_test = []
        self.pcap_to_be_preprocessed_first = []

        for pcap_dir in self.PCAP_FILES_DISSECTION_DIRS:
            for dirname, dirnames, filenames in os.walk('./' + pcap_dir):
                for filename in filenames:
                    # print("file: " + filename)
                    complete_filename = os.path.join(dirname, filename)

                    # case open wsn profile of pcap
                    if "openwsn_captures" in dirname:
                        self.pcap_for_test.append(
                            openwsn_profile_filter(complete_filename, os.path.join(TMPDIR, "filtered_%s" % filename))
                        )
                    # stack completely dissectable by ttproto
                    elif filename.endswith('.pcap'):
                        self.pcap_for_test.append(complete_filename)
                    else:
                        logging.warning('[dissector unittests] file ignored for dissection: %s' % complete_filename)

    def test_get_implemented_protocols(self):
        # Get implemented protocols and check their values
        implemented_protocols = get_dissectable_protocols()
        logging.info("implemented protos: " + str(implemented_protocols))
        self.assertEqual(type(implemented_protocols), list)
        self.assertGreater(len(implemented_protocols), 0)
        for prot in implemented_protocols:
            self.assertTrue(issubclass(prot, PacketValue))

    def test_that_it_can_dissect_all_pcaps(self):
        """
        this basicallyt test that the decoders dont raise any errors
        :return:
        """

        logging.info(
            "[dissector unittests] loaded %s .pcap files for dissection tests" % len(self.pcap_for_test))

        for p_file in self.pcap_for_test:
            logging.info('[dissector unittests] dissecting %s' % p_file)
            c = Capture(p_file)
            d = c.get_dissection()
            try:
                logging.debug('frame dissection: %s' % json.dumps(d, indent=4))
            except:
                logging.debug('frame dissection: %s' % d)

    def test_dissected_pcaps_returned_values_are_not_empty(self):
        """
        this test that the decoders dont raise any errors
        :return:
        """

        logging.info(
            "[dissector unittests] loaded %s .pcap files for dissection tests" % len(self.pcap_for_test))

        for p_file in self.pcap_for_test:
            logging.info('[dissector unittests] dissecting %s' % p_file)
            c = Capture(p_file)
            d = c.get_dissection(self.PROTO_CLASS_FILTER)
            if len(d) == 0:
                self.fail('got empty dissection for %s layer for .pcap %s' % (self.PROTO_CLASS_FILTER, p_file))
            logging.debug('frame dissection: %s' % json.dumps(d, indent=4))


# #################### Main run the tests #########################
if __name__ == '__main__':
    unittest.main()



from ..common import *


class TD_IPSO_3303_03(CoAPTestCase):
    """
testcase_id: TD_IPSO_3303_03 
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m 
configuration: LWM2M_CFG_01 
objective: Delete an Instance of Temperature Object
pre_conditions:
  - Device is registred with the Server
  - The current values of the Server Object (ID:1) Instance 0, are saved on the Server
  - The Client supports the Configuration C.3 (A superset of C.1 where server objects implements {Default Minimum PEriod, Default Maximum Period, Disable Timeout} additional resources)  
  - - LWM2M Server Object (ID = 1) Instance 0 with mandatory resources and Short Server ID = 1
  - - LWM2M Security Object (ID = 0) Instance 0 with mandatory resources and Bootstrap Server = False
  - - LWM2M Device Object (ID = 3) Instance 0 with mandatory resources. 
  - The Client supports IPSO Temperature object (ID:3303) with mandatory resources. 
sequence:
  - step_id: 'TD_IPSO_3303_03_step_01'
    type: stimuli
    node: lwm2m_server
    description:
      - 'LwM2M server sends a DELETE request (CoAP Delete) on Temperature object instance'
      - - Type = 0 (CON)
        - Code = 4 (DELETE)

  - step_id: 'TD_IPSO_3303_03_step_02'
    type: check
    description:
      - 'Sent DELETE request contains'
      - - Type=0 and Code=4
        - empty payload
        - URI-Path option= Location of Temperature object instance. 

  - step_id: 'TD_IPSO_3303_03_step_03'
    type: check
    description:
      - 'LwM2M client sends response containing'
      - - Code = 2.02 (Deleted)
        - empty payload. 

  - step_id: 'TD_IPSO_3303_03_step_04'
    type: verify
    node: lwm2m_server
    description:
      - 'LwM2M server indicates successful operation'


    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='delete')]

    def run(self):
        self.match('server', CoAP(type='con', code='delete', opt=self.uri()), 'fail')
        self.match('server', CoAP(pl=(b'')), 'fail')       

        self.next()

        self.match('client', CoAP(code=2.02), 'fail')
        self.match('client', CoAP(pl=(b'')), 'fail')

                



from ..common import *


class TD_COAP_CORE_20(CoAPTestCase):
    """
---
TD_COAP_CORE_20:
    cfg: CoAP_CFG_BASIC
    obj: Perform GET transaction containing the Accept option (CON mode)
    pre:
        - 'Server should provide a resource /multi-format which exists in two
            formats: '
        -   - text/plain;charset=utf-8
            - application/xml
    ref: '[COAP] 5.8.1, 5.10.5, 5.10.4'
    seq:
        -   n: client requests a resource in text format
        -   s: "Client is requested to send a confirmable GET request to server\
                \u2019s resource"
        -   c:
            - 'The request sent request by the client contains:'
            -   - Type = 0 (CON)
                - Code = 1 (GET)
                - "Client-generated Message ID (\u2794 CMID)"
                - "Client-generated Token (\u2794 CTOK)"
                - Option type = Accept, value = 0 (text/plain;charset=utf-8)
                - Uri-Path option "multi-format"
        -   c:
            - 'Server sends response containing:'
            -   - Code = 2.05 (Content)
                - Message ID = CMID, Token = CTOK
                - Option type = Content-Format, value = 0
                                (text/plain;charset=utf-8)
                - Payload = Content of the requested resource in
                            text/plain;charset=utf-8 format
        -   v: Client displays the response
        -   n: client requests a resource in xml format
        -   s: "Client is requested to send a confirmable GET request to server\
                \u2019s resource"
        -   c:
            - 'The request sent by the client contains:'
            -   - Type = 0 (CON)
                - Code = 1 (GET)
                - "Another client-generated Message ID \u2260 CMID (\u2794 CMID2)"
                - "Client-generated Token which may or may not be \u2260 CTOK\
                    \ (\u2794 CTOK2)"
                - Option type = Accept, value = 41 (application/xml)
        -   c:
            - 'Server sends response containing:'
            -   - Code = 2.05 (Content)
                - Message ID = CMID2, Token = CTOK2
                - Option type = Content-Format, value = 41 (application/xml)
            - Payload = Content of the requested resource in application/xml
                format
        -   v: Client displays the response
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            # Step 1 and Step 5 are the same stimulis.
            CoAP(type='con', code='get', opt=Opt(CoAPOptionUriPath("multi-format")))
        ]

    def run(self):

        # Step 2
        self.match("client", CoAP(type="con",
                                  code="get",
                                  opt=self.uri("/multi-format") if self.urifilter else Opt(CoAPOptionAccept())))

        self.match("client", CoAP(type="con",
                                  code="get",
                                  opt=Opt(CoAPOptionAccept(0))))
        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        self.next_skip_ack()
        # Step 3
        self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                  code=2.05,
                                  mid=CMID,
                                  tok=CTOK,
                                  opt=Opt(CoAPOptionContentFormat(0)),
                                  pl=Not(b"")))

        self.next_skip_ack(optional=True)
        # Step 6
        self.match("client", CoAP(type="con",
                                  code="get",
                                  opt=self.uri("/multi-format", CoAPOptionAccept(41))))
        CMID2 = self.coap["mid"]
        CTOK2 = self.coap["tok"]

        if CMID2 != b"":
            if CMID2 == CMID:
                self.set_verdict("fail", "Message ID should be different")
        if CTOK2 != b"":
            if CTOK2 == CTOK:
                self.set_verdict("fail", "Token should be different")

        self.next_skip_ack()
        # Step 7
        self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                  code=2.05,
                                  mid=CMID2,
                                  tok=CTOK2))

        if self.match("server", CoAP(pl=Not(b""))):
            self.match("server", CoAP(opt=Opt(CoAPOptionContentFormat(41)),), "fail")

        self.next_skip_ack(optional=True)
